/*
 * Device3D.java
 *
 * Created on Piatok, 2004, oktber 15, 19:34
 */

/**
 
 * @author  Marek Mizanin
 */

package graphics.device;

import java.util.*;
import graphics.math.*;
import graphics.device.Device2D_triangle;
import graphics.device.Material;
import graphics.device.Light;

public class Device3D extends Device2D_triangle
{
	// primitives
//	final public static int GL_POINTS = 1;
	final public static int GL_LINES = 2;
	final public static int GL_LINE_STRIP = 3;
	final public static int GL_LINE_LOOP = 4;
	final public static int GL_TRIANGLES = 5;
	final public static int GL_TRIANGLE_STRIP = 6;
	final public static int GL_TRIANGLE_FAN = 7;
	final public static int GL_LINE_POLYGON_MODE = 8;
	public int primitiveType = GL_LINE_STRIP;
	// render mode
	
	final public static int GL_FLAT = 0;
	final public static int GL_SMOOTH = 1;
//	final public static int TEXTURED = 2;
	// kombination
	//      | FLAT | SMOOTH==GOURAUD | TEXTURED
	// NO Z | F    | G               | T
	//    Z | FZ   | GZ              | TZ
	public int shadeModel = GL_FLAT;
	public boolean lighting = false;
	int light_ambient;
	public boolean lineMode = false;
	
	
	public Material material = new Material();	// front
	Vector lights = new Vector();		// vector lights
	v eye=new v();	// position of eye in object space
		
	public int lineWidth = 1;
	public int modifiLineWidth = 0;
	
	public mat view = new mat();		// modelview matrix
//	public float viewMulX,viewAddX;
//	public float viewMulY,viewAddY;
	public boolean ortho = false;
	
	// vertex position
	public v first;		// for GL_LINE_LOOP, GL_TRIANGLE_FAN
	public v next0;
	public v next1;
	
	// transformed position
	public vi actual_tr;
	public vi first_tr;
	public vi next0_tr;
	public vi next1_tr;
	// transformed position in eye space
	public v actual_tr_e;
	public v first_tr_e;
	public v next0_tr_e;
	public v next1_tr_e;
	
	// vertex color
//	public int actual_c;	// in Device2D
	public int first_c;
	public int next0_c;
	public int next1_c;
	// normal vector
	public v actual_n = new v();
	public v first_n = new v();
	public v next0_n = new v();
	public v next1_n = new v();
	
	public Device3D(){}
	public Device3D(int sizeX_, int sizeY_)
	{
		super(sizeX_, sizeY_);
	}
	
	/*
	 D2 - function is in Device2D, or Device2D_triangle class
	 GL - function is call from GL class
	 drawLineF (v a, v b)		// GL
	 drawLineF (vi a, vi b)									// D2
	 drawLineF (vi a, vi b, int width)						// D2
	 drawLineFZ(v a, v b)
	 drawLineFZ(vi a, vi b, float A, float B)				// D2
	 drawLineFZ(vi a, vi b, float A, float B, int width)	// D2
	 	 
	 drawLineG (v a, v b, int ac, int bc)		// GL
	 drawLineG (vi a, vi b, int ac, int bc)					// D2
	 drawLineG (vi a, vi b, int ac, int bc, int width)		// D2
	 drawLineGZ(v a, v b, int ac, int bc)
	 drawLineGZ(vi a, vi b, int ac, int bc, float A, float B)				// D2
	 drawLineGZ(vi a, vi b, int ac, int bc, float A, float B, int width)	// D2
	 
	 drawTriangleF (v a, v b, v c)						// GL
	 drawTriangleF (vi a, vi b, vi c, v ae, v be, v ce)	// GL
	 drawTriangleF (vi a, vi b, vi c)							// D2
	 drawTriangleFZ(v a, v b, v c)
	 drawTriangleFZ(vi a, vi b, vi c, v ae, v be, v ce)
	 drawTriangleFZ(vi a, vi b, vi c, v normal, float delta)	// D2
	 
	 drawTriangleG (v a, v b, v c, int ac, int bc, int cc)		// GL
	 drawTriangleG (v a, v b, v c, v an, v bn, v cn)			// GL
	 drawTriangleG (vi a, vi b, vi c, int ac, int bc, int cc, v ae, v be, v ce)	// GL
	 drawTriangleG (vi a, vi b, vi c, int ac, int bc, int cc)							// D2
	 
	 drawTriangleGZ(v a, v b, v c, int ac, int bc, int cc)
	 drawTriangleGZ(vi a, vi b, vi c, int ac, int bc, int cc, v ae, v be, v ce)
	 drawTriangleGZ(vi a, vi b, vi c, v normal, float delta, int ac, int bc, int cc)	// D2
	 */
	public vi eye2ScreenOrtho(v a)	// eye space to screen space with ortho projection
	{
		return new vi((int)(a.x*viewMulX + viewAddX),(int)(a.y*viewMulY + viewAddY));	
	}
	public vi eye2ScreenPersp(v a)	// eye space to screen space with perspective projection
	{
		float div = 1.f/-a.z;
		return new vi((int)(a.x*viewMulX*div + viewAddX),(int)(a.y*viewMulY*div + viewAddY));	
	}
	public vi eye2Screen(v a)
	{
		if(ortho)
			return eye2ScreenOrtho(a);
		else 
			return eye2ScreenPersp(a);
	}
	
	public void drawLineF(v a, v b)
	{
		if(useZBuffer)
			drawLineFZ(a,b);
		else
		{
			v ae = view.mul(a);
			v be = view.mul(b);
			if(ae.z>-0.1f || be.z>-0.1f)return;

			if(ortho)
			{
				if(lineWidth==1)
					drawLineF( eye2ScreenOrtho(ae), eye2ScreenOrtho(be));
				else
					drawLineF( eye2ScreenOrtho(ae), eye2ScreenOrtho(be), lineWidth);
			}
			else
			{
				if(lineWidth==1)
					drawLineF( eye2ScreenPersp(ae), eye2ScreenPersp(be));
				else
					drawLineF( eye2ScreenPersp(ae), eye2ScreenPersp(be), lineWidth);
			}
		}
	}
	
	public void drawLineG(v a, v b, int ac, int bc)
	{
		if(useZBuffer)
			drawLineGZ(a,b,ac,bc);
		else
		{
			v ae = view.mul(a);
			v be = view.mul(b);
			if(ae.z>-0.1f || be.z>-0.1f)return;
			if(ortho)
			{
				if(lineWidth==1)
					drawLineG( eye2ScreenOrtho(ae), eye2ScreenOrtho(be), ac, bc);
				else
					drawLineG( eye2ScreenOrtho(ae), eye2ScreenOrtho(be), ac, bc, lineWidth);		
			}
			else
			{
				if(lineWidth==1)
					drawLineG( eye2ScreenPersp(ae), eye2ScreenPersp(be), ac, bc);
				else
					drawLineG( eye2ScreenPersp(ae), eye2ScreenPersp(be), ac, bc, lineWidth);	
			}
		}
	}
	
	public void drawLineFZ(v a, v b)
	{
		float A,B;
		v ae = view.mul(a);
		v be = view.mul(b);
		if(ae.z>-0.1f || be.z>-0.1f)return;
		
		vi ai = eye2ScreenPersp(ae);
		vi bi = eye2ScreenPersp(be);

		if(Math.abs(bi.x-ai.x)>Math.abs(bi.y-ai.y)) // main axis is x, inc per 1
		{
			if((be.x-ae.x)==0)return;
			// a.z = A*a.x + B; // I.
			// b.z = A*b.x + B; // II.,  make II. - I.
			// A = (b.z-a.z)/(b.x-a.x)
			// B = a.z/(a*a.x)
			A = (be.z-ae.z)/(be.x-ae.x);
			B = be.z-A*ae.x;
		}
		else // main axis is y
		{
			if((be.y-ae.y)==0)return;
			A = (be.z-ae.z)/(be.y-ae.y);
			B = be.z-A*ae.y;
		}
		if(lineWidth==1)
			drawLineFZ(ai,bi,A,-1.f/B);
		else
			drawLineFZ(ai,bi,A,-1.f/B,lineWidth);
	}

	public void drawLineGZ(v a, v b, int ac, int bc)
	{
		float A,B;
		v ae = view.mul(a);
		v be = view.mul(b);
		if(ae.z>-0.1f || be.z>-0.1f)return;
		
		vi ai = eye2ScreenPersp(ae);
		vi bi = eye2ScreenPersp(be);
		
		if(Math.abs(bi.x-ai.x)>Math.abs(bi.y-ai.y)) // main axis is x, inc per 1
		{
			if((be.x-ae.x)==0)return;
			// a.z = A*a.x + B; // I.
			// b.z = A*b.x + B; // II.,  make II. - I.
			// A = (b.z-a.z)/(b.x-a.x)
			// B = a.z/(a*a.x)
			A = (be.z-ae.z)/(be.x-ae.x);
			B = be.z-A*ae.x;
		}
		else // main axis is y
		{
			if((be.y-ae.y)==0)return;
			A = (be.z-ae.z)/(be.y-ae.y);
			B = be.z-A*ae.y;
		}
		if(lineWidth==1)
			drawLineGZ(ai, bi, ac, bc, A, -1.f/B);
		else
			drawLineGZ(ai, bi, ac, bc, A, -1.f/B, lineWidth);
	}
	
	public void drawTriangleF(v a, v b, v c)
	{
		if(useZBuffer)
			drawTriangleFZ(a,b,c);
		else
		{
			v ae = view.mul(a);
			v be = view.mul(b);
			v ce = view.mul(c);
			if(ae.z>-0.1f || be.z>-0.1f || ce.z>-0.1f)return;
			if(ortho)
				drawTriangleF( eye2ScreenOrtho(ae), eye2ScreenOrtho(be), eye2ScreenOrtho(ce));
			else
				drawTriangleF( eye2ScreenPersp(ae), eye2ScreenPersp(be), eye2ScreenPersp(ce));
		}
	}
	public void drawTriangleF(vi a, vi b, vi c, v ae, v be, v ce)
	{
		if(useZBuffer)
			drawTriangleFZ(a,b,c,ae,be,ce);
		else
		{
			if(ae.z>-0.1f || be.z>-0.1f || ce.z>-0.1f)return;
			if(ortho)
				drawTriangleF( eye2ScreenOrtho(ae), eye2ScreenOrtho(be), eye2ScreenOrtho(ce));
			else
				drawTriangleF( eye2ScreenPersp(ae), eye2ScreenPersp(be), eye2ScreenPersp(ce));
		}
	}
	
	public void drawTriangleG(v a, v b, v c, int ac, int bc, int cc)
	{
		if(useZBuffer)
			drawTriangleGZ(a,b,c,ac,bc,cc);
		else
		{
			v ae = view.mul(a);
			v be = view.mul(b);
			v ce = view.mul(c);
			if(ae.z>-0.1f || be.z>-0.1f || ce.z>-0.1f)return;
			if(ortho)
				drawTriangleG( eye2ScreenOrtho(ae), eye2ScreenOrtho(be), eye2ScreenOrtho(ce), ac, bc, cc);
			else
				drawTriangleG( eye2ScreenPersp(ae), eye2ScreenPersp(be), eye2ScreenPersp(ce), ac, bc, cc);
		}
	}
	
	public void drawTriangleG(v a, v b, v c, v an, v bn, v cn)
	{
		drawTriangleG(a, b, c, computeLighting(a, an), computeLighting(b, bn), computeLighting(c, cn) );
	}
	
	public void drawTriangleG(vi a, vi b, vi c, int ac, int bc, int cc, v ae, v be, v ce)
	{
		if(useZBuffer)
			drawTriangleGZ(a,b,c,ac,bc,cc,ae,be,ce);
		else
		{
			if(ae.z>-0.1f || be.z>-0.1f || ce.z>-0.1f)return;
			if(ortho)
				drawTriangleG( eye2ScreenOrtho(ae), eye2ScreenOrtho(be), eye2ScreenOrtho(ce), ac, bc, cc);
			else
				drawTriangleG( eye2ScreenPersp(ae), eye2ScreenPersp(be), eye2ScreenPersp(ce), ac, bc, cc);
		}
	}
	
	// with z-buffer
	public void drawTriangleFZ(v a, v b, v c)
	{
		v ae = view.mul(a);
		v be = view.mul(b);
		v ce = view.mul(c);
		if(ae.z>-0.1f || be.z>-0.1f || ce.z>-0.1f)return;
		
		v normal = v.MakeNormal(ae,be,ce);
		float delta = - v.DOT3(normal,ae);
		
		drawTriangleFZ(eye2ScreenPersp(ae),eye2ScreenPersp(be),eye2ScreenPersp(ce), normal, delta);
	}
	public void drawTriangleFZ(vi a, vi b, vi c, v ae, v be, v ce)
	{
		if(ae.z>-0.1f || be.z>-0.1f || ce.z>-0.1f)return;
		v normal = v.MakeNormal(ae,be,ce);
		float delta = - v.DOT3(normal,ae);
		
		drawTriangleFZ(a, b, c, normal, delta);
	}
	
	public void drawTriangleGZ(v a, v b, v c, int ac, int bc, int cc)
	{
		// to eye space
		v ae = view.mul(a);
		v be = view.mul(b);
		v ce = view.mul(c);
		if(ae.z>-0.1f || be.z>-0.1f || ce.z>-0.1f)return;
		
		v normal = v.MakeNormal(ae,be,ce);
		float delta = - v.DOT3(normal,ae);
		
		drawTriangleGZ(eye2ScreenPersp(ae),eye2ScreenPersp(be),eye2ScreenPersp(ce), normal, delta, ac, bc, cc);
	}
	
	public void drawTriangleGZ(vi a, vi b, vi c, int ac, int bc, int cc, v ae, v be, v ce)
	{
		if(ae.z>-0.1f || be.z>-0.1f || ce.z>-0.1f)return;
		v normal = v.MakeNormal(ae,be,ce);
		float delta = - v.DOT3(normal,ae);
		
		drawTriangleGZ(a, b, c, normal, delta, ac, bc, cc);
	}
	
/*	public boolean lighting = false;
	int light_ambient;
	
	Material material = new Material();	// front
	Vector lights = new Vector();		// vector lights*/
	
	public void recomputeLightPositions()
	{
		mat view_inv = view.Inverse();
		eye = mat.mul(view_inv,new v(0,0,0));
		for(int i=0; i<lights.size(); i++)
		{
			Light l = (Light)lights.get(i);
			if(l.isObjectSpace)
			{
				l.position_obj_sp.set(l.position);
			}
			else
			{
				if(l.directional)
					l.position_obj_sp.set(mat.mul3x3(view_inv,l.position));
				else
					l.position_obj_sp.set(mat.mul(view_inv,l.position));
			}
			if(l.directional)
				l.position_obj_sp.Normalize();
		}
	}
	
	public int computeLighting(v pos, v normal)
	{
		if(material==null)
			return 0xffffff;
		int c = material.emission;
		
		// c += light_ambient*material.ambient;
		c = col.add(c,col.mul(light_ambient,material.ambient));
		
		for(int i=0; i<lights.size(); i++)
		{
			int cl=0;
			Light l = (Light)lights.get(i);
			
			// cl = light.ambient*material.ambient;
			cl = col.mul(l.ambient,material.ambient);
			
			if(l.directional)
			{
				// vp = l.pos - pos;
				float brightness = v.DOT3_clamp(l.position_obj_sp,normal);
				// cl += l.diffuse*material.diffuse*brightness;
				cl = col.add(cl,col.mul(col.mul(l.diffuse,material.diffuse),brightness));
				if(brightness!=0.f)
				{
					v ve = v.Normalize(v.sub(eye,pos));
					v h = v.Normalize(v.add(ve,l.position_obj_sp));
					brightness = (float)Math.pow(v.DOT3_clamp(h,normal),material.shininess);
					// cl += l.specular*material.specular*brightness;
					cl = col.add(cl,col.mul(col.mul(l.specular,material.specular),brightness));
				}				
			}
			else // point light
			{
				// vp = l.pos - pos;
				v vp = v.sub(l.position_obj_sp,pos);
				vp.Normalize();
				float brightness = v.DOT3_clamp(vp,normal);
				// cl += l.diffuse*material.diffuse*brightness;
				cl = col.add(cl,col.mul(col.mul(l.diffuse,material.diffuse),brightness));
				if(brightness!=0.f)
				{
					v ve = v.Normalize(v.sub(eye,pos));
					v h = v.Normalize(v.add(ve,vp));
					brightness = (float)Math.pow(v.DOT3_clamp(h,normal),material.shininess);
					// cl += l.specular*material.specular*brightness;
					cl = col.add(cl,col.mul(col.mul(l.specular,material.specular),brightness));
				}
			}
			// attenuation
			c = col.add(c,cl);
		}
		return c;
	}
}
