/*
 * GL.java
 *
 * Created on Nedee, 2004, oktber 24, 19:23
 */

package graphics.device;
import graphics.math.*;
import graphics.device.Device2D;
import graphics.device.Device3D;
/**
 *
 * @author  Marek Mizanin
 * GL - graphics library, all function with gl prefix
 */
public class GL extends Device3D
{
	public GL(){}
	public GL(int sizeX_, int sizeY_)
	{
		super(sizeX_, sizeY_);
		glSetPerspective(45.f);
	}
	
	public void glLoadViewMatrix(mat m)
	{
		view.set(m);
	}
	
	public void glSetPerspective(float fovy)	// use if z buffer is on
	{
		viewMulY = (float)(1.0/Math.tan(fovy * v.PI180 * 0.5));
		viewMulX = sizeX*0.5f*viewMulY*(float)sizeY/(float)sizeX;
		viewMulY *=-sizeY*0.5f;
		viewMulX_inv_n = -1.f/viewMulX;
		viewMulY_inv_n = -1.f/viewMulY;
		viewAddX = sizeX*0.5f;
		viewAddY = sizeY*0.5f;
		ortho = false;
	}
	public void glSetOrtho(float height)
	{
		height *= 0.5f;
		viewMulX = 0.5f*sizeY/height;
		viewMulY =-viewMulX;
		viewAddX = sizeX*0.5f;
		viewAddY = sizeY*0.5f;
		ortho = true;
		glDisableZBuffer();
	}
	
	public void glVertex3fv(v a)
	{
		switch(primitiveType)
		{
	//	case GL_POINTS:
	//		drawPoint(a);
	//		break;
		case GL_LINE_POLYGON_MODE:
			if(next0==null)
			{
				next0 = a;
				return;
			}
			actual_c = material.diffuse;
			drawLineF(next0, a); 
			next0 = a;
			break;
		case GL_LINES:
			if(shadeModel==GL_SMOOTH)
			{
				if(next0==null)
				{
					next0 = a;
					next0_c = actual_c;
					return;
				}
				drawLineG(next0, a, next0_c, actual_c); 
				next0 = null;
			}
			else  // GL_FTAT
			{
				if(next0==null)
				{
					next0 = a;
					return;
				}
				drawLineF(next0, a);
				next0 = null;
			}
			break;
		case GL_LINE_LOOP:
			if(first==null)
			{
				first = a;
				first_c = actual_c;
			}			
		case GL_LINE_STRIP:
			if(shadeModel==GL_SMOOTH)
			{
				if(next0==null)
				{
					next0 = a;
					next0_c = actual_c;
					return;
				}
				drawLineG(next0, a, next0_c, actual_c); 
				next0 = a;
				next0_c = actual_c;
			}
			else // GL_FLAT
			{
				if(next0==null)
				{
					next0 = a;
					return;
				}
				drawLineF(next0, a); 
				next0 = a;
			}
			break;
		case GL_TRIANGLES:
			if(shadeModel==GL_SMOOTH)
			{
				if(lighting)
				{
					if(next0==null)
					{
						next0 = a;
						next0_n.set(actual_n);
						return;
					}
					if(next1==null)
					{
						next1 = a;
						next1_n.set(actual_n);
						return;
					}
					drawTriangleG(next0, next1, a, next0_n, next1_n, actual_n);
					next0 = null;
					next1 = null;					
				}
				else // no lighting
				{
					if(next0==null)
					{
						next0 = a;
						next0_c = actual_c;
						return;
					}
					if(next1==null)
					{
						next1 = a;
						next1_c = actual_c;
						return;
					}
					drawTriangleG(next0, next1, a, next0_c, next1_c, actual_c);
					next0 = null;
					next1 = null;					
				}
			}
			else // GL_FLAT
			{
				if(lighting)
				{
					if(next0==null)
					{
						next0 = a;
						return;
					}
					if(next1==null)
					{
						next1 = a;
						return;
					}
					actual_c = computeLighting(a, actual_n);
					drawTriangleF(next0, next1, a);
					next0 = null;
					next1 = null;					
				}
				else // no lighting
				{
					if(next0==null)
					{
						next0 = a;
						return;
					}
					if(next1==null)
					{
						next1 = a;
						return;
					}
					drawTriangleF(next0, next1, a);
					next0 = null;
					next1 = null;					
				}				
			}
			break;
		case GL_TRIANGLE_STRIP:
			if(shadeModel==GL_SMOOTH)
			{
				if(lighting)
				{
					int actual_c_;
					if(next0==null)
					{
						next0 = a;
						next0_tr_e = view.mul(a);
						next0_tr = eye2Screen(next0_tr_e);
					//	next0_n.set(actual_n);
						next0_c = computeLighting(a, actual_n);
						return;
					}
					if(next1==null)
					{
						next1 = a;
						next1_tr_e = view.mul(a);
						next1_tr = eye2Screen(next1_tr_e);
					//	next1_n.set(actual_n);
						next1_c = computeLighting(a, actual_n);
						return;
					}
					actual_c_ = computeLighting(a, actual_n);
					actual_tr_e = view.mul(a);
					actual_tr = eye2Screen(actual_tr_e);
					drawTriangleG(next0_tr, next1_tr, actual_tr, next0_c, next1_c, actual_c_, next0_tr_e, next1_tr_e, actual_tr_e);
					next0 = next1;
					next0_tr = next1_tr;
					next0_tr_e = next1_tr_e;
					next0_c = next1_c;
				//	next0_n.set(next1_n);
					next1 = a;
					next1_tr = actual_tr;
					next1_tr_e = actual_tr_e;
					next1_c = actual_c_;
				//	next1_n.set(actual_n);
					cullBack = !cullBack;					
				}
				else // no lighting
				{
					if(next0==null)
					{
						next0 = a;
						next0_tr_e = view.mul(a);
						next0_tr = eye2Screen(next0_tr_e);
						next0_c = actual_c;
						return;
					}
					if(next1==null)
					{
						next1 = a;
						next1_tr_e = view.mul(a);
						next1_tr = eye2Screen(next1_tr_e);
						next1_c = actual_c;
						return;
					}
					actual_tr_e = view.mul(a);
					actual_tr = eye2Screen(actual_tr_e);
					drawTriangleG(next0_tr, next1_tr, actual_tr, next0_c, next1_c, actual_c, next0_tr_e, next1_tr_e, actual_tr_e);
					next0 = next1;
					next0_tr = next1_tr;
					next0_tr_e = next1_tr_e;
					next0_c = next1_c;
					next1 = a;
					next1_tr = actual_tr;
					next1_tr_e = actual_tr_e;
					next1_c = actual_c;
					cullBack = !cullBack;					
				}
			}
			else // GL_FLAT
			{
				if(lighting)
				{
					if(next0==null)
					{
						next0 = a;
						next0_tr_e = view.mul(a);
						next0_tr = eye2Screen(next0_tr_e);
						return;
					}
					if(next1==null)
					{
						next1 = a;
						next1_tr_e = view.mul(a);
						next1_tr = eye2Screen(next1_tr_e);
						return;
					}
					actual_c = computeLighting(a, actual_n);
					actual_tr_e = view.mul(a);
					actual_tr = eye2Screen(actual_tr_e);
					drawTriangleF(next0_tr, next1_tr, actual_tr, next0_tr_e, next1_tr_e, actual_tr_e);
					next0 = next1;
					next0_tr = next1_tr;
					next0_tr_e = next1_tr_e;
					next1 = a;
					next1_tr = actual_tr;
					next1_tr_e = actual_tr_e;
					cullBack = !cullBack;						
				}
				else // no lighting
				{
					if(next0==null)
					{
						next0 = a;
						next0_tr_e = view.mul(a);
						next0_tr = eye2Screen(next0_tr_e);
						return;
					}
					if(next1==null)
					{
						next1 = a;
						next1_tr_e = view.mul(a);
						next1_tr = eye2Screen(next1_tr_e);
						return;
					}
					actual_tr_e = view.mul(a);
					actual_tr = eye2Screen(actual_tr_e);
					drawTriangleF(next0_tr, next1_tr, actual_tr, next0_tr_e, next1_tr_e, actual_tr_e);
					next0 = next1;
					next0_tr = next1_tr;
					next0_tr_e = next1_tr_e;
					next1 = a;
					next1_tr = actual_tr;
					next1_tr_e = actual_tr_e;
					cullBack = !cullBack;					
				}				
			}
			break;
		case GL_TRIANGLE_FAN:
			if(shadeModel==GL_SMOOTH)
			{
				if(lighting)
				{
					int actual_c_;
					if(first==null)
					{
						first = a;
						first_tr_e = view.mul(a);
						first_tr = eye2Screen(first_tr_e);
					//	first_n.set(actual_n);
						first_c = computeLighting(a, actual_n);
					}
					if(next0==null)
					{
						next0 = a;
						next0_tr_e = view.mul(a);
						next0_tr = eye2Screen(next0_tr_e);
					//	next0_n.set(actual_n);
						next0_c = computeLighting(a, actual_n);
						return;
					}
					actual_c_ = computeLighting(a, actual_n);
					actual_tr_e = view.mul(a);
					actual_tr = eye2Screen(actual_tr_e);
					drawTriangleG(first_tr, next0_tr, actual_tr, first_c, next0_c, actual_c_, first_tr_e, next0_tr_e, actual_tr_e);
					next0 = a;
					next0_tr = actual_tr;
					next0_tr_e = actual_tr_e;
				//	next0_n.set(actual_n);
					next0_c = actual_c_;					
				}
				else // no lighting
				{
					if(first==null)
					{
						first = a;
						first_tr_e = view.mul(a);
						first_tr = eye2Screen(first_tr_e);
						first_c = actual_c;
					}
					if(next0==null)
					{
						next0 = a;
						next0_tr_e = view.mul(a);
						next0_tr = eye2Screen(next0_tr_e);
						next0_c = actual_c;
						return;
					}
					actual_tr_e = view.mul(a);
					actual_tr = eye2Screen(actual_tr_e);
					drawTriangleG(first_tr, next0_tr, actual_tr, first_c, next0_c, actual_c, first_tr_e, next0_tr_e, actual_tr_e);
					next0 = a;
					next0_tr = actual_tr;
					next0_tr_e = actual_tr_e;
					next0_c = actual_c;					
				}
			}
			else // GL_FLAT
			{
				if(lighting)
				{
					if(first==null)
					{
						first = a;
						first_tr_e = view.mul(a);
						first_tr = eye2Screen(first_tr_e);
					}
					if(next0==null)
					{
						next0 = a;
						next0_tr_e = view.mul(a);
						next0_tr = eye2Screen(next0_tr_e);
						return;
					}
					actual_c = computeLighting(a, actual_n);
					actual_tr_e = view.mul(a);
					actual_tr = eye2Screen(actual_tr_e);
					drawTriangleF(first_tr, next0_tr, actual_tr, first_tr_e, next0_tr_e, actual_tr_e);
					next0 = a;
					next0_tr = actual_tr;
					next0_tr_e = actual_tr_e;
				}
				else // no lighting
				{
					if(first==null)
					{
						first = a;
						first_tr_e = view.mul(a);
						first_tr = eye2Screen(first_tr_e);
					}
					if(next0==null)
					{
						next0 = a;
						next0_tr_e = view.mul(a);
						next0_tr = eye2Screen(next0_tr_e);
						return;
					}
					actual_tr_e = view.mul(a);
					actual_tr = eye2Screen(actual_tr_e);
					drawTriangleF(first_tr, next0_tr, actual_tr, first_tr_e, next0_tr_e, actual_tr_e);
					next0 = a;
					next0_tr = actual_tr;
					next0_tr_e = actual_tr_e;
				}				
			}			
			break;
		}
	}
	public void glVertex3f(float a, float b, float c)
	{
		glVertex3fv(new v(a,b,c));
	}

	public void glBegin(int type)
	{
		cullBack = cullBackSet;
		primitiveType = type;
		first=null;
		next0=null;
		next1=null;	
		if(lineMode)
			primitiveType = GL_LINE_POLYGON_MODE;
	}
	public void glEnd()
	{
		if(primitiveType == GL_LINE_LOOP)
		{
			if(shadeModel==GL_SMOOTH)
				drawLineG(next1, first, next1_c, first_c);
			else
				drawLineF(next1, first);
		}	
	}
	public void glColor3f(float r, float g, float b)
	{
		actual_c = col.make(r,g,b);
	}
	public void glColor3fv(int a)
	{
		actual_c = a;
	}
	public void glColor3fv(v a)
	{
		actual_c = col.make(a.x,a.y,a.z);
	}
	public void glNormal3f(float x, float y, float z)
	{
		actual_n.set(x,y,z);
	}
	public void glNormal3fv(v a)
	{
		actual_n.set(a);
	}
	
	public void glLineWidth(int width)
	{
		if(width<1)width=1;
		lineWidth = width;
		
	}
	public void glShadeModel(int model)
	{
		shadeModel = model;
	}
	public void glLightModelAmbient(int ambient)
	{
		light_ambient = ambient;
	}
	public void glEnableLighting(){lighting = true;}
	public void glDisableLighting(){lighting = false;}
	public void glMaterial(Material a){ material.set(a);}
	public void glAddLight(Light a)
	{
		lights.add(new Light(a));
	}
	public void glRemoveAllLights()
	{
		lights.clear();
	}
	public void glRecomputeLightPositions()
	{
		recomputeLightPositions();
	}
	public void glEnableCullFace(){ cullFace=true; }
	public void glDisableCullFace(){ cullFace=false; }
	public void glCullFaceBack(){ cullBackSet=true; }
	public void glCullFaceFront(){ cullBackSet=false; }
	public void glEnableZBuffer(){ useZBuffer=true; }
	public void glDisableZBuffer(){ useZBuffer=false; }
	public void glPolygonModeLine(){ lineMode=true;}
	public void glPolygonModeTriangle(){ lineMode=false;}
}
