/**
 * @(#)Lsystem_text.java
 *
 * Trieda pre nacitavanie L systemov agenerovanie retazcov
 *
 * @author Peter Ovcarik
 * @version 1.00 04/10/10
 */

package lsystem.generovanie; 
import java.util.*;
import javax.swing.*;
import java.lang.*;
import javax.naming.directory.*;
import java.io.*;
//import org.omg.CORBA.*; 

public class Lsystem_text
{
 	private class symbol
    {
        char name;							// symbol kontextu
        Vector name_param = new Vector();    // typu "String", parametre symbolu
        Vector hodnot = new Vector();        // typu "float", skutocne hodnoty parametrov
        int pocet_param;                    // pocet parametrov
    }

    private class z_production	// struktura popisujuca jedno pravidlo
    {
        char l;									// prepisovany symbol
        String	r = new String();				// prava strana pravidla
        Vector	param = new Vector();			// typu "String", formalne parametre lavej casti
        String	podmienka = new String();		// podmienka pre pouzitie pravidla
        float	prav;							// pravdepodobnost
        int		r_max_length;					// maximalna dlzka pravej strany, pouzivam to pre vypocet dlzky celeho retazca
        Vector	lavy_kontext = new Vector();	// typu "symbol", lavy kontext (retazec nalavo od prepisovaneho symbolu)
        Vector	pravy_kontext = new Vector();	// typu "symbol", pravy kontext (retazec napravo od prepisovaneho symbolu)
        Vector	zoznam_stoch = new Vector(); 	// typu "z_production", zoznam stochastickych pravidiel
        int 	je_stochasticke;
	
        z_production(z_production pr)
        {
            l = pr.l;
            r = pr.r;
            param = new Vector(pr.param);
            podmienka = pr.podmienka;
            prav = pr.prav;
            r_max_length = pr.r_max_length;
            for(int i=0; i<pr.lavy_kontext.size(); i++)		// skopiruje sa lavy kontext
            {
                symbol s = new symbol();
                lavy_kontext.addElement(s);
                symbol sym = (symbol)lavy_kontext.get(i);
                symbol sym1 = (symbol)pr.lavy_kontext.get(i);
			
                sym.name = sym1.name;
                sym.name_param = new Vector(sym1.name_param);
                sym.hodnot = new Vector(sym1.hodnot);
                sym.pocet_param = sym1.pocet_param;
            }
            for(int i=0; i<pr.pravy_kontext.size(); i++)		// skopiruje sa lavy kontext
            {
                symbol s = new symbol();
                pravy_kontext.addElement(s);
                symbol sym = (symbol)pravy_kontext.get(i);
                symbol sym1 = (symbol)pr.pravy_kontext.get(i);
			
                sym.name = sym1.name;
                sym.name_param = new Vector(sym1.name_param);
                sym.hodnot = new Vector(sym1.hodnot);
                sym.pocet_param = sym1.pocet_param;
            }
            zoznam_stoch = new Vector(pr.zoznam_stoch);
            je_stochasticke = pr.je_stochasticke;
        }
        
	    z_production(){}
    }

    private class z_constan		// struktura popisujuca konstanty
    {
        String	name = new String();    // meno konstatny
        float	x;                      // hodnota
    }

    private class stoch
    {
        int cislo_pravidla;
        float prav;
    }
    
    //////////////////////////////////////////////////////////////////////////
    float PI180 = (float)(Math.PI/180);
    String	axiom = new String();			// axiom
	public float	angle;                          // uhol
	Vector	prod = new Vector();			// typu "z_production", vektor produkcnych pravidiel
	public Vector	prod_hom = new Vector();		// typu "z_production", vektor produkcnych pravidiel pre homomorphism
	Vector	cons = new Vector();			// typu "z_constan", vektor konstant
	String	ignore_symbol = new String();	// symboly, ktore sa budu ignorovat pri kontextovych l-systemoch
    
	public int level;                       // level vysledneho retazca
	
	public StringBuffer r = new StringBuffer();	// vysledny retazec, bez aplikovania homomorphism-u
	public StringBuffer rh = new StringBuffer();	// vysledny retazec, s aplikovanim homomorphism-u
	
	public int		max_dlzka_retazca;	// nastavena v konstruktore na 1 000 000
	int		mod_generovania;	// nastavy sa na 0, ak nacitame retazec
	int		predpoklad_dlzky;	// pomocna premenna
	public boolean	je_parametricky;	// pomocna premenna, ktora udava ci dany L-system je parametridky
	
    public Lsystem_text()
	{
		max_dlzka_retazca = 10000000;
		mod_generovania = 1;
		axiom = "!(0.2)F";
		angle = 37.0f;

		z_production a = new z_production();

		a.l = 'F';
	//	a.r = "FF+[+F-F-F]-[-F+F+F]";
		a.r = "!(0.2)[++F-F!(0)][--F+F!(0)][^^F&F!(0)][&&F^F!(0)]FFF!(0)";
	//	a.r = "[++F-F][--F+F][^^F&F][&&F^F]FFF";
	//	a.r = "FF^[^F&F&F]&[&F^F^F]";

		a.r_max_length = a.r.length();
		prod.addElement(a);
		level = -1;
		r = new StringBuffer();
	}
	
    private int PreskocZatvorkyHranate(int i, String s)
	{
		int zat=1;		// lave zatvorky
		i++;
		for(; i<s.length()-1; i++)		// -1 aby sme potom netestovali mimo pola v teste s[i+1]==
		{
			if(s.charAt(i)=='[') zat++;
			if(s.charAt(i)==']') zat--;
			if(zat==0) break;			// nasla sa zatvorka parova k prvej otvaracej
		}
		i++;
		//if(i>=s.length()) i=s.length()-1;
		return i;
	}
	
	private int PreskocZatvorkyHranateR(int i, String s)
	{
		int zat=1;		
		i--;
		for(; i>=0; i--)		
		{
			if(s.charAt(i)==']') zat++;
			if(s.charAt(i)=='[') zat--;
			if(zat==0) break;		// nasla sa zatvorka parova k prvej zatvaracej
		}
		i--;
		if(i<0) i=0;
		return i;
	}
	
	private int Je_pravy_kontext(int i, z_production prod, String s, String p_kontext1, int pocet_symbolov1)
	{
		//JOptionPane.showMessageDialog(null,"i: "+String.valueOf(i)+", s: "+s+", p_kontext1: "+p_kontext1+", pocet_symbolov1: "+String.valueOf(pocet_symbolov1),"Chyba",JOptionPane.ERROR_MESSAGE);
		String p_kontext = new String(p_kontext1); 
		int pocet_symbolov = pocet_symbolov1;
	
		if(i<s.length()-1)
		{
			i++;
			int poc;
			for(poc=i; poc<s.length(); poc++)
			{
				if(s.charAt(poc)=='[')	poc = PreskocZatvorkyHranate(poc, s);
				if(poc<s.length() && s.charAt(poc)==']') break;
				if(pocet_symbolov >= prod.pravy_kontext.size()) break;
				else if(poc<s.length() && s.charAt(poc)!='[')
				{
					if(poc<s.length()&& s.charAt(poc)=='(')
					{
						for( ; poc<s.length() && s.charAt(poc)!=')'; poc++)
						{
							p_kontext+=s.charAt(poc);
						}
						if(poc<s.length()) p_kontext+=s.charAt(poc);
					}
					else if(poc<s.length())
					{
						p_kontext+=s.charAt(poc);
						pocet_symbolov++;
					}
				}
				if(poc<s.length() && s.charAt(poc)=='[') poc--;
			}
			if(poc<s.length()) if(s.charAt(poc)=='(')
			{
				for( ; poc<s.length() && s.charAt(poc)!=')'; poc++)
				{
					p_kontext+=s.charAt(poc);
				}
				if(poc<s.length()) p_kontext+=s.charAt(poc);
			}
	
			// porovnanie
			if(p_kontext.length()!=0) 
			{
				if(pocet_symbolov==prod.pravy_kontext.size())
				{
					int h,pocet_symb;
					symbol sym;
					Vector pom_pravy_kont = new Vector(); // vector<symbol> pom_pravy_kont;
					for(h=0,pocet_symb=0; h<p_kontext.length()-1; h++)
					{
						sym = new symbol();
						sym.pocet_param=0;
						sym.name_param.clear(); 
						sym.hodnot.clear();
						if(p_kontext.charAt(h)!='(' && p_kontext.charAt(h)!=')') sym.name=p_kontext.charAt(h);
						if(p_kontext.charAt(h+1)=='(')
						{
							String par = new String();		
							String hodnot = new String();
							h+=2;
							for( ; p_kontext.charAt(h)!=')' && h<p_kontext.length(); h++)	par+=p_kontext.charAt(h);
							for(int j=0; j<par.length(); j++)
							{
								if(par.charAt(j)==',' && hodnot.length()>0)
								{
									sym.hodnot.addElement(Float.valueOf(hodnot));
									sym.pocet_param++;
									hodnot = new String();
								}
								else hodnot+=par.charAt(j);
							}
							if(hodnot.length()>0)
							{
								sym.hodnot.addElement(Float.valueOf(hodnot));
								sym.pocet_param++; 
							}
						}
						pom_pravy_kont.addElement(sym);
					}
					if(h==p_kontext.length()-1 && p_kontext.charAt(h)!='(' && p_kontext.charAt(h)!=')')
					{
						sym = new symbol();
						sym.hodnot.clear();
						sym.name_param.clear();
						sym.pocet_param=0;
						sym.name=p_kontext.charAt(h);
						pom_pravy_kont.addElement(sym);
					}
					
					int ok=0;
					for(h=0; h<pocet_symbolov; h++)
					{
	      				//JOptionPane.showMessageDialog(null,String.valueOf(pocet_symbolov)+" "+String.valueOf(pom_pravy_kont.size()),"Chyba",JOptionPane.ERROR_MESSAGE);
						symbol pom_symbol1 = (symbol)prod.pravy_kontext.get(h);
						symbol pom_symbol2 = (symbol)pom_pravy_kont.get(h);
						if(pom_symbol1.name==pom_symbol2.name && pom_symbol1.pocet_param==pom_symbol2.pocet_param)
						{
							ok++;
						}
					}
					if(ok==pocet_symbolov)
					{
						for(h=0; h<prod.pravy_kontext.size(); h++)
						{
							symbol pom_symbol = (symbol)prod.pravy_kontext.get(h);
							symbol pom_symbol1 = (symbol)pom_pravy_kont.get(h);
							for(int j=0; j<pom_symbol.pocet_param; j++)
							{
								pom_symbol.hodnot.addElement(pom_symbol1.hodnot.get(j));
							}
						}
						return 1;
					}
				}
			}
		
			//ak neuspelo porovnanie
			for( ; i<s.length() && s.charAt(i)!='[' && s.charAt(i)!=']'; i++)
			{
				if(s.charAt(i)=='(')
				{
					for( ; i<s.length() && s.charAt(i)!=')'; i++)
					{
						p_kontext1+=s.charAt(i);
					}
					if(i<s.length()) p_kontext1+=s.charAt(i);
				}
				else if(i<s.length())
				{
					p_kontext1+=s.charAt(i);
					pocet_symbolov1++;
				}
				if(pocet_symbolov1>=prod.pravy_kontext.size())
				{
					if(i<s.length()-1) if(s.charAt(i+1)=='(')
					{
						i++;
						for( ; i<s.length() && s.charAt(i)!=')'; i++)
						{
							p_kontext1+=s.charAt(i);
						}
						if(i<s.length()) p_kontext1+=s.charAt(i);
						i++;
					}
					break;
				}
			}
			
			if(i<s.length() && s.charAt(i)=='[')  return Je_pravy_kontext(i, prod, s, p_kontext1, pocet_symbolov1);
			int j;
			if(i<s.length() && s.charAt(i)==']') j=i-1;
			else j=i;
			int p;
			for(p=0; j>=0 && j<s.length() && s.charAt(j)!='['; j--)
			{
				if(s.charAt(j)==']')
				{
					j = PreskocZatvorkyHranateR(j, s); 
					j++;
				}
				else 
				{
					if(s.charAt(j)==')')
					{
						for( ; j<s.length() && s.charAt(j)!='('; j--);
						//j--;						
					}
					else if(j>=0)
					{
						p++;
					}
				}
			}
		
			if(j>=0)
			{
				if((pocet_symbolov1-p)<0) p=0;
				int y,nul;
				for(y=p_kontext.length()-1, nul=p; y>=0 && nul>0; y--)
				{
					if(p_kontext.charAt(y)==')')
					{
						for( ; y>=0 && p_kontext.charAt(y)!='('; y--);
					}
					else nul--;
				}
				p_kontext1=p_kontext1.substring(0, y+1);
			}
			if(i<s.length() && s.charAt(i)==']') return Je_pravy_kontext(i, prod, s, p_kontext1, pocet_symbolov1-p);
			else
			{
				i++;
				for( ;i<s.length() && s.charAt(i)!=']'; i++)
				{
					if(s.charAt(i)=='[')
					{
						i = PreskocZatvorkyHranate(i, s); 
						i--;
					}
					if(s.charAt(i)=='(') for( ; s.charAt(i)!=')' && i<s.length(); i++);
				}
				return Je_pravy_kontext(i, prod, s, p_kontext1, pocet_symbolov1-p);
			}
		}	
		return 0;
	}
	
    private int Vyber_cislo_z_prav(Vector prav)
    {
        float rand_cislo0_1=(float)Math.random();
        float suma=0,predch_prav=0;
        int cislo_sady=0;
	
        for(int i=0; i<prav.size(); i++)
        {
            suma += ((Float)prav.get(i)).floatValue();
            if(rand_cislo0_1>=predch_prav && rand_cislo0_1<=suma)
            {
                cislo_sady=i;
                break;
            }
            predch_prav = ((Float)prav.get(i)).floatValue();
        }
            return cislo_sady;
    }
        
	public int Generuj(int pozadovany_level)		// vygeneruje retazec s danym levelom, vrati vygenerovany level
	{
		je_parametricky=false;
		int size = 0;				// velkost retazca
		String a = new String();                // retazec a bude zdrojovy, z nim budeme porovnavat pravidla
		int level_a = 0;			// level zdrojoveho retazca
		
		if(mod_generovania==0) return 0;
		if(level>0 && level<pozadovany_level)	// ak vygenerovany retazec r ma level < ako pozadovany level
		{							// mozme ho pouzit
			a = r.toString();		// skopirujeme ho do a
			level_a = level;		// nastavime pociatocny level
			r.delete(0,r.length());	// zmazeme stary retazec
		}
		else						// inak pouzijeme axiom
		{
			r = new StringBuffer();	// uvolnime pamet zabranu r
			a = new String();
			for(int i=0; i<axiom.length(); i++)
			{
				int zat=0;
				if(axiom.charAt(i)=='(' || (zat!=0 && axiom.charAt(i)==','))
				{
					a += axiom.charAt(i);
					if(axiom.charAt(i)=='(') zat=1;
					int j;
					for(j=i+1; j<axiom.length(); j++)
					{
						if(axiom.charAt(j)=='(')zat++;
						if(axiom.charAt(j)==')')zat--;
						if(axiom.charAt(j)==',')break;
						if(zat==0)break;
					}
					// j ukazuje na poslednu zatvorku, alebo ciarku
					String vyraz = new String(axiom.substring(i+1,j));
					String temp = new String();
					Vector param = new Vector();	// vector<string> param
					Vector hodnot = new Vector();	// vector<float> hodnot
			
					float value = Vypocitaj_vyraz(vyraz, param, hodnot);
					/*if( (float)fabs(value)<0.1f)
						sprintf( temp, "%.5f", value);
					else
						gcvt( value, 10,temp);*/	// konvertujeme hodnotu vyrazu, retazec
					temp=String.valueOf(value);
					//sprintf( temp, "%.5f", value);
					je_parametricky=true;
					a+= temp;	
					i = j-1;	// i sa este zvysi
				}
				else a += axiom.charAt(i);
			}
			
			level=0;
			if(pozadovany_level==0)				// ak pozadovany level je 0 mozme rovno skoncit
			{
				r.append(a);
				if(prod_hom.size()>0) Aplikuj_homomorphism();
				return 0;
			}
			level_a = 0;				// pouzivame axiom
		}
		
		for(int i=level; i<pozadovany_level; i++)		// pre vsetky levely az po konecny
		{
			size=a.length();					// vypocitame potrebnu velkost retazca
			int j;
			for(j=0; j<a.length(); j++)		// pre kazdy znak
			{
				char ch=a.charAt(j);
				for(int pr=0; pr<prod.size(); pr++)	// pre vsetky pravidla
				{
					z_production pom_prod = (z_production)prod.get(pr);
					if(pom_prod.l==ch) size+=pom_prod.r_max_length-1;	// pripocitame k size velkost pravej strany pravidla-1
				}
			}
			
			if(size>max_dlzka_retazca)		// ak je retazec vecsi ako maxmimalna velkost skoncime
			{
				JOptionPane.showMessageDialog(null,"Vygenerovany retazec presiahol nastavenu maximalnu dlzku.","Varovanie",JOptionPane.WARNING_MESSAGE);
				r.delete(0,r.length());
				r.append(a);				// do vysledneho retazca skopirujeme posledny vygenerovany
				level = /*level_a*/i;
				if(prod_hom.size()>0) Aplikuj_homomorphism();
				return level;//level_a;
			}
			r.ensureCapacity(size);		// rezervujeme si pamet o velkosti size
		
			predpoklad_dlzky = size;
		
			int param = 0;
			for(j=0; j<a.length(); j++)				// pre kazdy znak
			{
				if(a.charAt(j)=='(')			// zatvorky preskakujeme, jedna sa o samostatne zatvorky typu "() F", alebo "f(1) ()"
				{
					int zat=1;			// zaciname	za jednou lavou zatvorkov
					j++;
					for(; j<a.length() && zat>0; j++)	// prechadzame retazec pokial nanajdeme pravu uzatvaraciu zatvorku
					{
						if(a.charAt(j)=='(') zat++;
						else if(a.charAt(j)==')') zat--;
					}
					// j ukazuje na znak nasledujuci po ')'
					j--;continue;	// j sa nezmeni
				}
				param=0;
				int k;
				if(j+1<a.length() && a.charAt(j+1)=='(' )		// ak ma dany znak parametre, zistime pocet parametrov
				{
					k=j+2;
					param++;		// zvysime pocet parametrov ( na 1)
					// spocitame parametre
					int zat=1;			// zaciname za jednou lavou zatvorkov
					for(; k<a.length() && zat>0; k++)	// prechadzame retazec pokial nanajdeme parovu uzatvaraciu zatvorku
					{
						if(a.charAt(k)==',') param++;		// kazda ciarka znamena dalsi parameter
						if(a.charAt(k)=='(') zat++;			// zvysime pocet lavych zatvoriek
						else if(a.charAt(k)==')') zat--;	// znizime pocet lavych zatvoriek
					}
				}
				int naslo_sa=0;		// pravidlo
				z_production prod_pom = new z_production();
				
				for(int pr=0; pr<prod.size(); pr++)		// pre vsetky pravidla
				{
					//z_production pom_prod1 = (z_production)prod.get(pr);
					z_production pom_prod1 = new z_production((z_production)prod.get(pr));
					if(pom_prod1.l==a.charAt(j) && pom_prod1.param.size()==param)    // pravidlo je mozne pouzit iba
					{																// ak sa zhoduje znak a pocet parametrov
						if(pom_prod1.je_stochasticke==0) prod_pom=pom_prod1;    // ak nie je stochasticke
						else          // je stochasticke
						{
							stoch st = new stoch();                         						// stochasticke pravidlo
							Vector vst = new Vector();                       // vector<stoch> vst
                                                        
							if(pom_prod1.zoznam_stoch.size()!=0)	
							{
                                st.cislo_pravidla=0;
								if(pom_prod1.prav>=0) st.prav=pom_prod1.prav;
                                else st.prav = 0;
								vst.addElement(st);
                                for(int g=0; g<pom_prod1.zoznam_stoch.size(); g++)
								{
                                    st = new stoch();
                                    st.cislo_pravidla=g+1;
                                    z_production pom_zoznam = (z_production)pom_prod1.zoznam_stoch.get(g);
                                    if(pom_zoznam.prav>=0) st.prav=pom_zoznam.prav;
                                    else st.prav = 0;
                                    vst.addElement(st);
								}
                                
                                float sucet_prav=0;
                                for(int g=0; g<vst.size(); g++)
								{
                                    sucet_prav += ((stoch)vst.get(g)).prav;
								}
                                if(sucet_prav <= 0) continue;
								
								Vector pravdepod = new Vector();
                                for(int g=0; g<vst.size(); g++)
								{
                                    pravdepod.addElement(new Float( ((stoch)vst.get(g)).prav/sucet_prav) );
								}
                                //////////////////////////////////
                                                                
                                int cislo_pravidla = Vyber_cislo_z_prav(pravdepod);
                                if(cislo_pravidla == 0) prod_pom=pom_prod1;
                                else prod_pom = (z_production)pom_prod1.zoznam_stoch.get(cislo_pravidla-1);
							}
							else
							{
								float pom_prav=pom_prod1.prav;		// je len jedno pravidlo
								if(pom_prav>1) pom_prav=1;
								if(pom_prav<0) pom_prav=0;
								if(pom_prav>0 && pom_prav>Math.random()) prod_pom=pom_prod1;
								else continue;
							}
						}
// test laveho kontextu
						int suhlasi_l_kontext=0;
						StringBuffer l_kontext = new StringBuffer();
						if(prod_pom.lavy_kontext.size()!=0)
						{
							StringBuffer pom2 = new StringBuffer();
							pom2.append(a.substring(0, j));
							if(ignore_symbol.length()!=0)
							{
								for(int ps=0; ps<pom2.length(); ps++)
								{
									for(int pi=0; pi<ignore_symbol.length(); pi++)
									{
										if(pom2.charAt(ps)==ignore_symbol.charAt(pi))
										{
											pom2.delete(ps,ps+1);
											if(ps<pom2.length() && pom2.charAt(ps)=='(')
											{
												int x;
												for(x=ps; x<pom2.length() && pom2.charAt(x)!=')'; x++)
												{
													pom2.delete(x,x+1);
													x--;
												}
												pom2.delete(x,x+1);
											}
											ps--;
											break;
										}
									}
								}
							}
							int pocet_symbolov=0;
							if(pom2.length()>0)
							{
								int sk=0;
								int poc;
								for(poc=pom2.length()-1; poc>=0; poc--)
								{
									if(pom2.charAt(poc)==']') poc = PreskocZatvorkyHranateR(poc, pom2.toString());
									if(pocet_symbolov>=prod_pom.lavy_kontext.size()) {sk=1; break;}
									else if(pom2.charAt(poc)!='[' && pom2.charAt(poc)!=']')
									{
										if(pom2.charAt(poc)==')')
										{
											for( ; poc>=0 && pom2.charAt(poc)!='('; poc--)
											{
												l_kontext.insert(0,pom2.charAt(poc));
											}
											if(poc>=0) l_kontext.insert(0,pom2.charAt(poc));
										}
										else if(poc>=0)
										{
											l_kontext.insert(0,pom2.charAt(poc));
											pocet_symbolov++;
										}
									}
									if(pom2.charAt(poc)==']') poc++;
								}
								
								if(poc>=0 && pom2.charAt(poc)==')' && sk==0)
								{
									for( ; poc>=0 && pom2.charAt(poc)!='('; poc--)
									{
										l_kontext.insert(0,pom2.charAt(poc));
									}	
									if(poc>=0) l_kontext.insert(0,pom2.charAt(poc));
								}
							}
	
    						if(l_kontext.length()!=0)
							{
								if(pocet_symbolov==prod_pom.lavy_kontext.size())
								{
									symbol sym;
									Vector pom_lavy_kont = new Vector();	// vector<symbol> pom_lavy_kont
									int ii; int pocet_symb;
									for(ii=0,pocet_symb=0; ii<l_kontext.length()-1; ii++)
									{
										sym = new symbol();
										sym.pocet_param=0;
										sym.name_param.clear();
										sym.hodnot.clear();
										if(l_kontext.charAt(ii)!='(' && l_kontext.charAt(ii)!=')') sym.name=l_kontext.charAt(ii);
										if(l_kontext.charAt(ii+1)=='(')
										{
											//sym.name=lavy_kontext[i];
											String par = new String();		// retazec obsahujuci formalne parametre
											String hodnot = new String();
											ii+=2;
											for( ; ii<l_kontext.length() && l_kontext.charAt(ii)!=')'; ii++)	par+=l_kontext.charAt(ii);
											for(int f=0; f<par.length(); f++)
											{
												if(par.charAt(f)==',' && hodnot.length()>0)
												{
													//sym.name_param.push_back(hodnot);
													sym.hodnot.addElement(Float.valueOf(hodnot));
													//JOptionPane.showMessageDialog(null,String.valueOf(sym.hodnot.get(0)),"Chyba",JOptionPane.ERROR_MESSAGE);
													sym.pocet_param++;
													hodnot = new String();
												}
												else hodnot+=par.charAt(f);
											}
											if(hodnot.length()>0) {sym.hodnot.addElement(Float.valueOf(hodnot));sym.pocet_param++;}
										}
										//JOptionPane.showMessageDialog(null,hodnot,"Chyba",JOptionPane.ERROR_MESSAGE);
										pom_lavy_kont.addElement(sym);
									}
									if(ii==l_kontext.length()-1 && l_kontext.charAt(ii)!='(' && l_kontext.charAt(ii)!=')')
									{
										sym = new symbol();
										sym.hodnot.clear();
										sym.name_param.clear();
										sym.pocet_param=0;
										sym.name=l_kontext.charAt(ii);
										pom_lavy_kont.addElement(sym);
									}

									// tu dalej if...
									int ok=0;
									for(int gg=0; gg<pocet_symbolov; gg++)
									{
										symbol lav_pom_kont = (symbol)prod_pom.lavy_kontext.get(gg);
										symbol lav_pom_kont1 = (symbol)pom_lavy_kont.get(gg);
										if(lav_pom_kont.name==lav_pom_kont1.name && lav_pom_kont.pocet_param==lav_pom_kont1.pocet_param)
										{
											ok++;
											for(int f=0; f<lav_pom_kont.pocet_param; f++)
											{
												lav_pom_kont.hodnot.addElement((Float)lav_pom_kont1.hodnot.get(f));
												//JOptionPane.showMessageDialog(null,String.valueOf(lav_pom_kont.hodnot.size()),"Chyba",JOptionPane.ERROR_MESSAGE);
											}
										}
									}
									if(ok==pocet_symbolov) suhlasi_l_kontext=1;
								}
								//else suhlasi_l_kontext=0;
							}
							//else suhlasi_l_kontext=0;
						}
						else suhlasi_l_kontext=1;
					
// test praveho kontextu				
						int suhlasi_p_kontext=1;
						if(prod_pom.pravy_kontext.size()!=0)
						{
							String pom = new String();
							StringBuffer pom1 = new StringBuffer();
							
							int p;
							for(p=j+1; p<a.length() && a.charAt(p)!=']'; p++)
							{
								if(a.charAt(p)=='[') { p = PreskocZatvorkyHranate(p,a); p--; }
							}
							pom1 = new StringBuffer();
							pom1.append(a.substring(j,p));
							if(ignore_symbol.length()!=0)
							{
								for(int ps=0; ps<pom1.length(); ps++)
								{
									for(int pi=0; pi<ignore_symbol.length(); pi++)
									{
										if(pom1.charAt(ps)==ignore_symbol.charAt(pi))
										{
											pom1.delete(ps,ps+1);
											if(pom1.charAt(ps)=='(')
											{
												int x;
												for(x=ps; x<pom1.length() && pom1.charAt(x)!=')'; x++)
												{
													pom1.delete(x,x+1);
													x--;
												}
												pom1.delete(x,x+1);
											}
											ps--;
											break;
										}
									}
								}
							}
						
							if(prod_pom.param.size()!=0)
							{
                                if(pom1.charAt(1)=='(')
                                {
                                    int x;
                                    for(x=1; x<pom1.length() && pom1.charAt(x)!=')'; x++)
                                    {
                                        pom1.delete(x,x+1);
                                        x--;
                                    }
                                    pom1.delete(x,x+1);
                                }
                            }
							suhlasi_p_kontext = Je_pravy_kontext(0, prod_pom, pom1.toString(), pom, 0);
						}
						//JOptionPane.showMessageDialog(null,String.valueOf(suhlasi_l_kontext)+String.valueOf(suhlasi_p_kontext),"Chyba",JOptionPane.ERROR_MESSAGE);
						if(suhlasi_l_kontext==1 && suhlasi_p_kontext==1)
						{
							if(param==0)		// ak nie su parametre
							{
								Vector parametre = new Vector();	// vector<float> parametre
								for(int ii=0; ii<prod_pom.lavy_kontext.size(); ii++)
								{
									symbol lav_kont = (symbol)prod_pom.lavy_kontext.get(ii);
									for(int o=0; o<lav_kont.pocet_param; o++)
									{
										//JOptionPane.showMessageDialog(null,(String)lav_kont.name_param.get(o),"Chyba",JOptionPane.ERROR_MESSAGE);
										prod_pom.param.addElement((String)lav_kont.name_param.get(o));
										parametre.addElement((Float)lav_kont.hodnot.get(o));
									}
								}
								//z_production pro = (z_production)prod.get(pr);
								//JOptionPane.showMessageDialog(null,String.valueOf(pro.param.size()),"Chyba",JOptionPane.ERROR_MESSAGE);
								for(int ii=0; ii<prod_pom.pravy_kontext.size(); ii++)
								{
									symbol prav_kont = (symbol)prod_pom.pravy_kontext.get(ii);
									for(int o=0; o<prav_kont.pocet_param; o++)
									{
										prod_pom.param.addElement((String)prav_kont.name_param.get(o));
										parametre.addElement((Float)prav_kont.hodnot.get(o));
									}
								}
								if(prod_pom.podmienka.length()==0)
								{
									// pridame retazec vytvoreny z pravej strany pravidla, tak, ze sa vyhodnotia vsetky vyrazy
									r.append(Vytvor_retazec(prod_pom.r, prod_pom.param, parametre)); naslo_sa=1;
								}
								else if(Vypocitaj_vyraz(prod_pom.podmienka, prod_pom.param, parametre)>0 )
								{	r.append(Vytvor_retazec(prod_pom.r, prod_pom.param, parametre)); naslo_sa=1;}
							}
							else				// su parametre
							{
								int kk=j+2,l=j+2;	// k,l ukazuju na zaciatok skut. parametrov
								int zat=1;			// 1 lava zatvorka
								Vector parametre = new Vector();	// vector<float> parametre
								int najdenych=0;	// najdenych parametrov
								for(; najdenych<param && l<a.length(); l++)		// pokial nenajdeme vsetky parametre, l ukazuje na znak
								{
									if(a.charAt(l)=='(') zat++;			// zvysime pocet lavych zatvoriek
									else if(a.charAt(l)==')') zat--;	// znizime pocet lavych zatvoriek
									if(a.charAt(l)==',' || zat==0)		// ak sa najde ',' alebo ukoncovacia zatvorka
									{
										String s = a.substring(kk,l);	// string obsahujuci cislo
										parametre.addElement(Float.valueOf(s));	// pridame skutocny parameter
										kk=l+1;					// dalsi zacina az za ','
										najdenych++;			// zvysime pocet najdenych parametrov
									}
								}
								for(int ii=0; ii<prod_pom.lavy_kontext.size(); ii++)
								{
									symbol lav_kont = (symbol)prod_pom.lavy_kontext.get(ii);
									for(int o=0; o<lav_kont.name_param.size(); o++)
									{
										//JOptionPane.showMessageDialog(null,(String)lav_kont.name_param.get(o),"Chyba",JOptionPane.ERROR_MESSAGE);
										prod_pom.param.addElement((String)lav_kont.name_param.get(o));
										parametre.addElement((Float)lav_kont.hodnot.get(o));
									}
								}
								for(int ii=0; ii<prod_pom.pravy_kontext.size(); ii++)
								{
									symbol prav_kont = (symbol)prod_pom.pravy_kontext.get(ii);
									for(int o=0; o<prav_kont.pocet_param; o++)
									{
										prod_pom.param.addElement((String)prav_kont.name_param.get(o));
										parametre.addElement((Float)prav_kont.hodnot.get(o));
									}
								}
								if(prod_pom.podmienka.length()==0)
								{	r.append(Vytvor_retazec(prod_pom.r, prod_pom.param, parametre)); naslo_sa=1;}
								else if(Vypocitaj_vyraz(prod_pom.podmienka, prod_pom.param, parametre)>0 )
								{	r.append(Vytvor_retazec(prod_pom.r, prod_pom.param, parametre)); naslo_sa=1;}
							}
							if(naslo_sa!=0) break;
						}
					}
				}// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
				
				if(naslo_sa==0)	//r+=a[j];// inak pridame povodny znak, t.j. ak sa nenaslo ziadne pravidlo
				{
					r.append(a.charAt(j));
					if(param!=0)
					{
						for(int kk=j+1; kk<a.length() && a.charAt(kk-1)!=')'; kk++)
						{
							r.append(a.charAt(kk));
						}
					}
				}
			}
			if(i<pozadovany_level-1)		// ak v tomto kroku este nekoncime
			{
				a = new String();	// zmazeme a
				a = r.toString();	// vymenime retazce
				r = new StringBuffer();
			}
			level++;			// zvysime level
		}
		
		if(prod_hom.size()>0) Aplikuj_homomorphism();
		return level;
	}
	
	public void Nacitaj(String in1)
	{
		String in=in1;
		
		int use_homomorphism=0;
		prod.clear();
		prod_hom.clear();
		level=-1;
		r = new StringBuffer();
        rh = new StringBuffer();
		cons.clear();
		ignore_symbol = new String();
		mod_generovania=1;
		Vector riad = new Vector();	// vector<string> riad;
		String p = new String();
		int d;

		// nakuskujeme retazec do riadkov
		for(int i=0; i<in.length(); i++)
		{
			if(in.charAt(i)=='\r') continue;	// navrat vozika ingnorujeme
			if(in.charAt(i)=='\n')				// riadok sa skoncil pridame do vektora
			{
				riad.addElement(p);
				p = new String();
			}
			else p+=in.charAt(i);					// vytvarame p
		}
		if(p.length()>0) riad.addElement(p);	// pridame posledny riadok

		int orezuj=0;
		for(int i=0; i<riad.size(); i++)
		{
			String riadok = (String)riad.get(i);
			int pos = riadok.indexOf("//*");
			if(pos!=-1) {riadok = riadok.substring(0, pos); riad.set(i,riadok);}
			if(orezuj!=0)
			{
				int pos1 = riadok.indexOf("*/");
				if(pos1==-1)		// nenaslo sa
				{
					riadok = new String();
					riad.set(i,"");
				}
				else				// naslo sa
				{
					riadok = riadok.substring(pos1+2,riadok.length());
					riad.set(i,riadok);
					i--;			// sposoby, ze tento riadok sa bude preverovat este raz
					orezuj = 0;
				}
				
				continue;
			}
			// tu sa dostaneme iba ak sa neorezuje
			int pos2 = riadok.indexOf("/*");
			if(pos2!=-1)		// naslo sa ?
			{
				String a = riadok.substring(0, pos2);
				String b = riadok.substring(pos2+2, riadok.length());
				int pos1 = b.indexOf("*/");
				if(pos1==-1)		// nenaslo sa
				{
					orezuj = 1;
					riadok = a;		// berieme iba to co je do / *
					riad.set(i,riadok);
				}
				else				// naslo sa
				{
					b = b.substring(pos1+2, b.length());		// retazec b bude od * /
					orezuj = 0;					// viac neorezujeme, nasli sme * /
					riadok = a+b;				// vysledok je to co bolo pred a to co bolo po bloku
					riad.set(i,riadok);
					i--;						// sposoby, ze tento riadok sa bude preverovat este raz
				}
			}
			
		}
		
		for(int k=0; k<riad.size(); k++)	// cez vsetky riadky
		{
			String riadok = (String)riad.get(k);
			for(d=0; d<riadok.length() && riadok.charAt(d)==' '; d++);		// vynechame medzery
			if(d==riadok.length())continue;				// ak sme na konci riadku, prejdeme na dalsi riadok
			
			String substr_angle = riadok.substring(d,Math.min(d+6,riadok.length()));
			String substr_define = riadok.substring(d,Math.min(d+7,riadok.length()));
			String substr_homomorph = riadok.substring(d,Math.min(d+12,riadok.length()));
			String substr_homo = riadok.substring(d,Math.min(d+4,riadok.length()));
			String substr_ignore = riadok.substring(d,Math.min(d+7,riadok.length()));
			String substr_krizik = riadok.substring(d,Math.min(d+1,riadok.length()));
			String substr = riadok.substring(d,Math.min(d+6,riadok.length()));
			if(substr.compareTo("axiom ")==0)
			{
				p = riadok.substring(d+6,riadok.length());
				// kontrola zatvoriek
				int okruh=0;
				int ok=1;
				for(int i=0; i<p.length(); i++)
				{
					switch(p.charAt(i))
					{
					case '(': okruh++; break;
					case ')': okruh--; if(okruh<0)ok=0; break;
					}
				}
				if(okruh!=0)ok=0;
				if(ok==0)
				{	
					String text = new String("Zatvorky v axiome nie su v poriadku na "+String.valueOf(k+1)+". riadku.\nCely riadok je ignorovany.");
      				JOptionPane.showMessageDialog(null,text,"Chyba",JOptionPane.ERROR_MESSAGE);
					continue;		// zatvorky nie su v poriadku !
				}
				// kontrola hranatych zatvoriek
				okruh=0;
				ok=1;
				for(int i=0; i<p.length(); i++)
				{
					switch(p.charAt(i))
					{
					case '[': okruh++; break;
					case ']': okruh--; if(okruh<0)ok=0; break;
					}
				}
				if(okruh!=0)ok=0;
				if(ok==0)
				{
					String text = new String("Hranate zatvorky v axiome nie su v poriadku na "+String.valueOf(k+1)+". riadku.\nCely riadok je ignorovany.");
      				JOptionPane.showMessageDialog(null,text,"Chyba",JOptionPane.ERROR_MESSAGE);
					continue;		// zatvorky nie su v poriadku !
				}
				axiom = new String();
				axiom = p;
			}
			else if(substr_angle.compareTo("angle ")==0)
			{
				p = riadok.substring(d+6,riadok.length());
                try {angle = Float.parseFloat(p);}
                catch (Exception e) {angle = 37.f;}
                
				//if(angle==0.0f) angle = 90.f;
			}
			
			else if(substr_define.compareTo("define ")==0)	// definovane konstanty
			{
				z_constan a = new z_constan();
				p = riadok.substring(d+7, riadok.length());
				int i;
				for(i=0; i<p.length() && p.charAt(i)!='='; i++)
				{
					if(p.charAt(i)==' ') continue;
					a.name+=p.charAt(i);
				}
				i++;
				if(i>=p.length())continue;
				p = p.substring(i,p.length());		// orezeme vsetko po = vratane '='
				while( p.length()>0 && p.charAt(0)==' ')p = p.substring(1,p.length());	// orezeme ' '
				// co zostalo je ciselne vyjadrena konstanta
				a.x = Float.valueOf(p).floatValue();
				cons.addElement(a);
			}
			else if(substr_homomorph.compareTo("homomorphism")==0)	// definovane konstanty
			{
				use_homomorphism = 1;
			}
			else if(substr_homo.compareTo("homo")==0)	// definovane konstanty
			{
				use_homomorphism = 1;
			}
			else if(substr_ignore.compareTo("ignore ")==0)
			{
				ignore_symbol = riadok.substring(d+7,riadok.length());
			}
			else if(substr_krizik.compareTo("#")==0)		// za # sa nachadza produkcne pravidlo
			{
				z_production a = new z_production();
				a.prav = 1.f;
			
				// zbavime retazec medzier, vynechavame prvy znak '#'
				p = new String();
				for(int i=d+1; i<riadok.length(); i++) if(riadok.charAt(i)!=' ') p+=riadok.charAt(i);

				{	// kontrola zatvoriek
					int okruh=0;
					int ok=1;
					for(int i=0; i<p.length(); i++)
					{
						switch(p.charAt(i))
						{
						case '(': okruh++; break;
						case ')': okruh--; if(okruh<0) ok=0; break;
						}
					}
					if(okruh!=0)ok=0;
					if(ok==0)
					{
						String text = "Zatvorky nie su v poriadku na "+String.valueOf(k+1)+". riadku.\nCely riadok je ignorovany.";
      					JOptionPane.showMessageDialog(null,text,"Chyba",JOptionPane.ERROR_MESSAGE);
						continue;		// zatvorky nie su v poriadku !
					}
				}
				
				if(p.length()==0) continue;	// ak je retazec nulovy koniec
// lavy kontext			
				int je_lavy_kontext=0;
				String lavy_kontext = new String();
				for(int i=2; i<p.length() && p.charAt(i-2)!=':' && (p.charAt(i-2)!='-' || p.charAt(i-2)!='>'); i++)
				{
					if(p.charAt(i-2)=='<' && p.charAt(i-1)!='(' && p.charAt(i-1)!=':' && p.charAt(i-1)!=')' && (p.charAt(i-1)!='-' || p.charAt(i)!='>'))
					{
						a.l=p.charAt(i-1);
						if(i!=2) lavy_kontext=p.substring(0, i-2);
						je_lavy_kontext=1;
						p=p.substring(i,p.length());
						break;
					}
				}

				if(lavy_kontext.length()!=0)
				{
					int okruh=0;
					int ok=1;
					for(int i=0; i<lavy_kontext.length(); i++)
					{
						switch(lavy_kontext.charAt(i))
						{
							case '(': okruh++; break;
							case ')': okruh--; if(okruh<0)ok=0; break;
						}
					}
					if(okruh!=0) ok=0;
					if(ok==0)
					{
						String text = "Zatvorky v lavom kontexte nie su v poriadku.\nLavy kontext bude ignorovany.";
      					JOptionPane.showMessageDialog(null,text,"Chyba",JOptionPane.ERROR_MESSAGE);
						je_lavy_kontext=0;
					}
					else
					{
						symbol sym; 
						int i;
						for(i=0; i<lavy_kontext.length()-1; i++)
						{
							sym = new symbol();
							sym.pocet_param=0;
							sym.name_param.clear();
							sym.hodnot.clear();
							if(lavy_kontext.charAt(i)!='(' && lavy_kontext.charAt(i)!=')') sym.name=lavy_kontext.charAt(i);
							if(lavy_kontext.charAt(i+1)=='(')
							{
								//sym.name=lavy_kontext[i];
								String par = new String();		// retazec obsahujuci formalne parametre
								String name = new String();
								i+=2;
								for( ; lavy_kontext.charAt(i)!=')' && i<lavy_kontext.length(); i++)	par+=lavy_kontext.charAt(i);
								for(int j=0; j<par.length(); j++)
								{
									if(par.charAt(j)==',' && name.length()>0)
									{
										sym.name_param.addElement(name);
										//sym.hodnot.push_back(0);
										sym.pocet_param++;
										name = new String();
									}	
									else name+=par.charAt(j);
								}
								if(name.length()!=0) {sym.name_param.addElement(name); ;sym.pocet_param++;}
							}
							a.lavy_kontext.addElement(sym);
						}
						if(i==lavy_kontext.length()-1 && lavy_kontext.charAt(i)!='(' && lavy_kontext.charAt(i)!=')')
						{
							sym = new symbol();
							sym.hodnot.clear();
							sym.name_param.clear();
							sym.pocet_param=0;
							sym.name=lavy_kontext.charAt(i);
							a.lavy_kontext.addElement(sym);
						}
					}
				}
				if(je_lavy_kontext==0) 
				{
					a.l = p.charAt(0);	// znak lavej strany
					p = p.substring(1,p.length());
				}
				//p = p.substr(1);			// skratime retazec o 1 znak
				
/////////////////////  koniec laveho kontextu				
				if(p.length()!=0 && p.charAt(0)=='(')
				{
					String par = new String();		// retazec obsahujuci formalne parametre
					String name = new String();

					int i;
					for(i=1; p.charAt(i)!=')';i++)	par+=p.charAt(i);
					p = p.substring(i+1,p.length());	// skratime retaze o zatvorku
					
					for( i=0; i<par.length(); i++)
					{
						if(par.charAt(i)==',' && name.length()>0)
						{
							a.param.addElement(name);
							name = new String();
						}
						else name+=par.charAt(i);
					}
					if(name.length()!=0) a.param.addElement(name);
				}
		
				if(p.length()==0) continue;
////// pravy kontext			
				String pravy_kontext = new String();
				if(p.charAt(0)=='>')
				{
					p=p.substring(1,p.length());
					int i;
					for(i=1; i<p.length() && p.charAt(i-1)!=':' && (p.charAt(i-1)!='-' || p.charAt(i)!='>'); i++)
					{
						pravy_kontext+=p.charAt(i-1);
					}
					p=p.substring(i-1,p.length());
				}
				
				if(pravy_kontext.length()!=0)
				{
					int okruh=0;
					int ok=1;
					for(int i=0; i<pravy_kontext.length(); i++)
					{
						switch(pravy_kontext.charAt(i))
						{
							case '(': okruh++; break;
							case ')': okruh--; if(okruh<0)ok=0; break;
						}
					}
					if(okruh!=0) ok=0;
					if(ok==0)
					{	
						JOptionPane.showMessageDialog(null,"Zatvorky v pravom kontexte nie su v poriadku.\nLavy kontext bude ignorovany.","Chyba",JOptionPane.ERROR_MESSAGE);
					}
					else
					{
						symbol sym;
						int i;
						for(i=0; i<pravy_kontext.length()-1; i++)
						{
							sym = new symbol();
							sym.pocet_param=0;
							sym.name_param.clear();
							sym.hodnot.clear();
							if(pravy_kontext.charAt(i)!='(' && pravy_kontext.charAt(i)!=')') sym.name=pravy_kontext.charAt(i);
							if(pravy_kontext.charAt(i+1)=='(')
							{
								String par = new String();		// retazec obsahujuci formalne parametre
								String name = new String();
								i+=2;
								for( ; pravy_kontext.charAt(i)!=')' && i<pravy_kontext.length(); i++)	par+=pravy_kontext.charAt(i);
								for(int j=0; j<par.length(); j++)
								{
									if(par.charAt(j)==',' && name.length()>0)
									{
										sym.name_param.addElement(name);
										//sym.hodnot.push_back(0);
										sym.pocet_param++;
										name = new String();
									}
									else name+=par.charAt(j);
								}
								if(name.length()!=0) {sym.name_param.addElement(name); sym.pocet_param++;}
							}
							a.pravy_kontext.addElement(sym);
						}
						if(i==pravy_kontext.length()-1 && pravy_kontext.charAt(i)!='(' && pravy_kontext.charAt(i)!=')')
						{
							sym = new symbol();
							sym.hodnot.clear();
							sym.name_param.clear();
							sym.pocet_param=0;
							sym.name=pravy_kontext.charAt(i);
							a.pravy_kontext.addElement(sym);
						}
					}
				}
//////// koniec praveho kontextu
				if(p.charAt(0)==':')	// podmienkovy vyraz
				{
					String pod = new String();
					p = p.substring(1,p.length());	// orezeme :
					int i;
					for(i=1; i<p.length() && (p.charAt(i-1)!='-' || p.charAt(i)!='>'); i++);
					if(i>=p.length())continue;
					a.podmienka = p.substring(0, i-1);		// i ukazuje na >
					p = p.substring(i-1,p.length());			// orezeme podmienkovy vyraz
				}
				int i;
				for(i=1; i<p.length() && (p.charAt(i-1)!='-' || p.charAt(i)!='>'); i++);
				i++;	// kedze i ukazuje na > musime i este zvysit
				if(i>=p.length())continue;
				p = p.substring(i,p.length());	// orezeme vsetko po a vratane "->"
				if(p.charAt(0)=='(')		// stochasticke pravidlo
				{
					int zat=1;
					for(i=1; i<p.length(); i++)
					{
						if(p.charAt(i)=='(')zat++;
						if(p.charAt(i)==')')zat--;
						if(zat==0) break;
					}
					// i ukazuje na poslednu zatvaraciu zatvorku  (1)
					
					String prav = p.substring(1,i);
					Vector empty = new Vector();	// vector<string> empty;
					Vector emptyf = new Vector();	// vector<float> emptyf;
					a.prav = Vypocitaj_vyraz(prav, empty, emptyf);
                                        a.je_stochasticke=1;
					i++;		// posunieme sa na prvy znak za )
					if(i>=p.length()) continue;
					p = p.substring(i,p.length());
				}
				else a.je_stochasticke=0;
				if(p.length()==0) continue;
				a.r = p;		// zostala nam prava strana
				int zat=0;
				a.r_max_length=0;
				for( i=0; i<a.r.length(); i++)
				{
					if(a.r.charAt(i)=='(')
					{
						if(zat==0) a.r_max_length+=4;		// zatvorka + cislo
						zat++;
						continue;
					}
					if(zat!=0 && a.r.charAt(i)==',')
					{
						a.r_max_length+=4;			// ciarka + cislo
						continue;
					}
					if(a.r.charAt(i)==')')zat--;
					a.r_max_length++;
				}
				if(use_homomorphism!=0) prod_hom.addElement(a);
				else
				{
					a.zoznam_stoch.clear();
					if(a.je_stochasticke==1)
					{
						int ok=0;
						for(i=0; i<prod.size(); i++)		
						{
							z_production prod_pomocny = (z_production)prod.get(i);
							if(prod_pomocny.je_stochasticke==1 && a.l==prod_pomocny.l && a.param.size()==prod_pomocny.param.size())
							{
								prod_pomocny.zoznam_stoch.addElement(a);
								ok=1;
								break;
							}
						}
						if(ok==0) prod.addElement(a);
					}
					else prod.addElement(a);
				}
			}
		}
		for(int i=0; i<axiom.length(); i++)
		if(axiom.charAt(i)==' ')axiom = axiom.substring(0,i)+axiom.substring(i+1,axiom.length());
		Vector param = new Vector();	// vector<string> param
		Vector hodnot = new Vector();	// vector<float>hodnot
		axiom = Vytvor_retazec(axiom, param, hodnot);

		Vector pom_prod = new Vector();	// vector<z_production> pom_prod
	
		for(int i=0; i<prod.size(); i++)		// kvoli vyssej priorite kontextovych pravidiel
		{
			z_production produc = (z_production)prod.get(i);
			if(produc.je_stochasticke==1)
			{
				pom_prod.addElement(produc);	// vsetky pravidla, ktore su stochasticke sa vlozia na zaciatok pola
			}
		}// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		for(int i=0; i<prod.size(); i++)		// kvoli vyssej priorite kontextovych pravidiel
		{
			z_production produc = (z_production)prod.get(i);
			if(produc.je_stochasticke==0 && (produc.lavy_kontext.size()!=0 || produc.pravy_kontext.size()!=0))
			{
				pom_prod.addElement(produc);	// vsetky pravidla, ktore maju kontexty sa vlozia do stredu pola
			}
		}
		for(int i=0; i<prod.size(); i++)		
		{
			z_production produc = (z_production)prod.get(i);
			if(produc.je_stochasticke==0 && produc.lavy_kontext.size()==0 && produc.pravy_kontext.size()==0)
			{
				pom_prod.addElement(produc);	// vsetky ostatne sa vlozia za kontextove pravidla
			}
		}
		
		prod.clear();
		prod=pom_prod;
				
		//z_production zz = (z_production)((z_production)prod.get(0)).zoznam_stoch.get(1);
		//symbol sus = (symbol)zz.lavy_kontext.get(1);
		//String t = new String(); t+=sus.name;
		//JOptionPane.showMessageDialog(null,zz.r,"Chyba",JOptionPane.ERROR_MESSAGE);
	}	
	
	private String Vytvor_retazec(String pr, Vector param1, Vector hodnot1) // param - vector String, hodnot - vector float
	{
		Vector param = new Vector(param1);
		Vector hodnot = new Vector(hodnot1);
		if(param.size()>0) 
		{
			//JOptionPane.showMessageDialog(null,"pr="+pr+", param[0]="+param.get(0)+", hodtnot[0]"+hodnot.get(0),"Chyba",JOptionPane.ERROR_MESSAGE);	
		}
		String r = new String();
		String temp = new String();

		int zat=0,a=0,b=0;
		for(int i=0; i<pr.length(); i++)		// prechazame cely retazec pr
		{
			if(pr.charAt(i)=='(')	// ak najdeme lavu zatvorku
			{
				if(zat==0)	// ak je to prva zatvorka
				{
					a=i+1;		// do a dame index prveho znaku za znakom '('
					r+='(';		// do vyslednuho retazaca pridame znak '('
				}
				zat++;			// zvysime pocet lavych zatvoriek
			}
			else if(pr.charAt(i)==')')	// ak najdeme pravu zatvorku
			{
				zat--;		// znizime pocet lavych zatvoriek
				if(zat==0)	// ak je to posledna zatvorka
				{
					String s = new String();
					s = pr.substring(a,i);		// s je retazec, ktory je v zatvorkach
					//JOptionPane.showMessageDialog(null,s,"Chyba",JOptionPane.ERROR_MESSAGE);
					float value = Vypocitaj_vyraz(s, param, hodnot);
					// konvertujeme hodnotu vyrazu, retazec
					temp = String.valueOf(value);	// sprintf( temp, "%.5f", value);
					je_parametricky=true;
					r+= temp;		// pridame retacez do r
					r+=')';
				}
			}
			else if(zat!=0 && pr.charAt(i)==',')		// ak najdeme ciarku, vyhodnotime vyraz medzi od indexu a az po ciarku
			{
				String s = new String();
				s = pr.substring(a,i);

				float value = Vypocitaj_vyraz(s, param, hodnot);
				// konvertujeme hodnotu vyrazu, retazec
				temp = String.valueOf(value);	// sprintf( temp, "%.5f", value);
				je_parametricky=true;
				r+= temp;
				r+=',';
				a=i+1;
			}
			else 
			if(zat==0)
			{
				r+=pr.charAt(i);	// ak nieje zatvorka pridavame znak po znaku do r
			}
		}
		return r;
	}
	
	private int PreskocZatvorky(int i, String s)
	{
		int zat=1;		// lave zatvorky
		i++;
		for(; i<s.length()-1; i++)		// -1 aby sme potom netestovali mimo pola v teste s[i+1]==
		{
			if(s.charAt(i)=='(') zat++;
			if(s.charAt(i)==')') zat--;
			if(zat==0) break;		// nasla sa zatvorka parova k prvej otvaracej
		}
		return i;
	}
	
	// vypocita vyraz s, param su nazvy premennych a hodnot su hodnoty premennych
	public float Vypocitaj_vyraz(String s, Vector param1, Vector hodnot1) // param - vector string, hodnot - vector float
	{
		Vector param = new Vector(param1);
		Vector hodnot = new Vector(hodnot1);
		
		String l = new String();
		String r = new String();
		float left,right;

		if(s.length()==0) return 0.f;

		if(s.charAt(0)=='(' && s.charAt(s.length()-1)==')')		// potrebujeme odstranit zatvorky
		{
			// musime osetrit pripad "() ()"
			int zat=0;
			for(int i=1; i<s.length()-1; i++)
			{
				if(s.charAt(i)=='(') zat++;
				if(s.charAt(i)==')') zat--;
				if(zat<0) break;			// ak nastane pripad taky, ze "(  ) (  )"
			}
			if(zat==0) s = s.substring(1, s.length()-1);	// odoberieme zatvorky
		}

		// ||		opertor logickho stu
		for(int i=0; i<s.length()-1; i++)	// prechadzame cely retazec, okrem posledneho znaku
		{

			if(s.charAt(i)=='(') i = PreskocZatvorky(i, s);	// vyraz v zatvorkach ma vysiu prioritu, preto sa bude vyhodnocovat neskor
			if(s.charAt(i)=='|' && s.charAt(i+1)=='|')	// ||		opertor logickho stu
			{
				l = s.substring(0, i);					// lava strana, pociatocny index 0, dlzka i
				r = s.substring(i+2, s.length());	// prava strana od i+2 po koniec
				left = Vypocitaj_vyraz(l, param, hodnot);
				right = Vypocitaj_vyraz(r, param, hodnot);
				if(left>0.f || right>0.f) return 1.0f;
				else return 0.f;
			}
		}

		// &&		opertor logickho sinu
		for(int i=0; i<s.length()-1; i++)	// prechadzame cely retazec, okrem posledneho znaku
		{
			if(s.charAt(i)=='(') i = PreskocZatvorky(i, s);	// vyraz v zatvorkach ma vysiu prioritu, preto sa bude vyhodnocovat neskor
			if(s.charAt(i)=='&' && s.charAt(i+1)=='&')	// &&		opertor logickho sinu
			{
				l = s.substring(0, i);					// lava strana, pociatocny index 0, dlzka i
				r = s.substring(i+2, s.length());	// prava strana od i+2 po koniec
				left = Vypocitaj_vyraz(l, param, hodnot);
				right = Vypocitaj_vyraz(r, param, hodnot);
				if(left>0.f && right>0.f)return 1.0f;
				else return 0.f;
			}
		}

		// !		unrny opertor logickej negcie
		for(int i=0; i<s.length()-1; i++)	// prechadzame cely retazec, okrem posledneho znaku
		{
			if(s.charAt(i)=='(') i = PreskocZatvorky( i, s);	// vyraz v zatvorkach ma vysiu prioritu, preto sa bude vyhodnocovat neskor
			if(s.charAt(i)=='!' && s.charAt(i+1)!='=')	// !		unrny opertor logickej negcie
			{
			//	l = s.substr(0, i);					// lava strana, pociatocny index 0, dlzka i
				r = s.substring(i+1, s.length());	// prava strana od i+1 po koniec
			//	left = Vypocitaj_vyraz( l, param, hodnot);
				right = Vypocitaj_vyraz(r, param, hodnot);
				if(right>0.f)return 1.0f;
				else return 0.f;
			}
		}

		// ==,!=,<,>,>=,<= relan opertory
		for(int i=0; i<s.length()-1; i++)	// prechadzame cely retazec, okrem posledneho znaku
		{
			if(s.charAt(i)=='(') i = PreskocZatvorky(i, s);	// vyraz v zatvorkach ma vysiu prioritu, preto sa bude vyhodnocovat neskor
			if(s.charAt(i)=='=' && s.charAt(i+1)=='=')	// ==	opertor rovnosti
			{
				l = s.substring(0, i);					// lava strana, pociatocny index 0, dlzka i
				r = s.substring(i+2, s.length());	// prava strana od i+2 po koniec
				left = Vypocitaj_vyraz(l, param, hodnot);
				right = Vypocitaj_vyraz(r, param, hodnot);
				if(left==right)return 1.0f;
				else return 0.f;
			}
			if(s.charAt(i)=='!' && s.charAt(i+1)=='=')	// !=	opertor nerovnosti
			{
				l = s.substring(0, i);					// lava strana, pociatocny index 0, dlzka i
				r = s.substring(i+2, s.length());	// prava strana od i+2 po koniec
				left = Vypocitaj_vyraz(l, param, hodnot);
				right = Vypocitaj_vyraz(r, param, hodnot);
				if(left!=right)return 1.0f;
				else return 0.f;
			}
			if(s.charAt(i)=='<' && s.charAt(i+1)!='=')	// <	opertor mensi
			{
				l = s.substring(0, i);					// lava strana, pociatocny index 0, dlzka i
				r = s.substring(i+1, s.length());	// prava strana od i+1 po koniec
				left = Vypocitaj_vyraz(l, param, hodnot);
				right = Vypocitaj_vyraz(r, param, hodnot);
				if(left<right)return 1.0f;
				else return 0.f;
			}
			if(s.charAt(i)=='>' && s.charAt(i+1)!='=')	// >	opertor vacsi
			{
				l = s.substring(0, i);					// lava strana, pociatocny index 0, dlzka i
				r = s.substring(i+1, s.length());	// prava strana od i+1 po koniec
				left = Vypocitaj_vyraz(l, param, hodnot);
				right = Vypocitaj_vyraz(r, param, hodnot);
				if(left>right)return 1.0f;
				else return 0.f;
			}
			if(s.charAt(i)=='>' && s.charAt(i+1)=='=')	// >=	opertor vacsi alebo rovnasa
			{
				l = s.substring(0, i);					// lava strana, pociatocny index 0, dlzka i
				r = s.substring(i+2, s.length());	// prava strana od i+2 po koniec
				left = Vypocitaj_vyraz(l, param, hodnot);
				right = Vypocitaj_vyraz(r, param, hodnot);
				if(left>=right)return 1.0f;
				else return 0.f;
			}
			if(s.charAt(i)=='<' && s.charAt(i+1)=='=')	// <=	opertor mensi alebo rovnasa
			{
				l = s.substring(0, i);					// lava strana, pociatocny index 0, dlzka i
				r = s.substring(i+2, s.length());	// prava strana od i+2 po koniec
				left = Vypocitaj_vyraz(l, param, hodnot);
				right = Vypocitaj_vyraz(r, param, hodnot);
				if(left<=right)return 1.0f;
				else return 0.f;
			}
		}

		// +,-		opertory stania a odtania
		for(int i=0; i<s.length(); i++)	// prechadzame cely retazec, okrem posledneho znaku
		{
			if(s.charAt(i)=='(') i = PreskocZatvorky( i, s);	// vyraz v zatvorkach ma vysiu prioritu, preto sa bude vyhodnocovat neskor
			if(s.charAt(i)=='+')	// 	+	opertor stania
			{
				l = s.substring(0, i);					// lava strana, pociatocny index 0, dlzka i
				r = s.substring(i+1, s.length());	// prava strana od i+1 po koniec
				left = Vypocitaj_vyraz(l, param, hodnot);
				right = Vypocitaj_vyraz(r, param, hodnot);
				return left+right;
			}
			if(s.charAt(i)=='-')	// 	-	opertor odtania
			{
				l = s.substring(0, i);					// lava strana, pociatocny index 0, dlzka i
				r = s.substring(i+1, s.length());	// prava strana od i+1 po koniec
				left = Vypocitaj_vyraz(l, param, hodnot);
				right = Vypocitaj_vyraz(r, param, hodnot);
				return left-right;
			}
		}
	
		// *,/,%		opertory nsobenia,delenia a zvyku po delen
		for(int i=0; i<s.length(); i++)	// prechadzame cely retazec, okrem posledneho znaku
		{
			if(s.charAt(i)=='(') i = PreskocZatvorky( i, s);	// vyraz v zatvorkach ma vysiu prioritu, preto sa bude vyhodnocovat neskor
			if(s.charAt(i)=='*')	// 	*	opertor nsobenia
			{
				l = s.substring(0, i);					// lava strana, pociatocny index 0, dlzka i
				r = s.substring(i+1, s.length());	// prava strana od i+1 po koniec
				left = Vypocitaj_vyraz(l, param, hodnot);
				right = Vypocitaj_vyraz(r, param, hodnot);
				return left*right;
			}
			if(s.charAt(i)=='/')	// 	-	opertor delenia
			{
				l = s.substring(0, i);					// lava strana, pociatocny index 0, dlzka i
				r = s.substring(i+1, s.length());	// prava strana od i+1 po koniec
				left = Vypocitaj_vyraz(l, param, hodnot);
				right = Vypocitaj_vyraz(r, param, hodnot);
				return left/right;
			}
			if(s.charAt(i)=='%')	// 	%	opertor zvyku po delen
			{
				l = s.substring(0, i);					// lava strana, pociatocny index 0, dlzka i
				r = s.substring(i+1, s.length());	// prava strana od i+1 po koniec
				left = Vypocitaj_vyraz(l, param, hodnot);
				right = Vypocitaj_vyraz(r, param, hodnot);
				return (float)Math.abs(Math.IEEEremainder(left,right));
			}
		}

		// ^	opertor umocovania
		for(int i=0; i<s.length(); i++)	// prechadzame cely retazec, okrem posledneho znaku
		{
			if(s.charAt(i)=='(') i = PreskocZatvorky( i, s);	// vyraz v zatvorkach ma vysiu prioritu, preto sa bude vyhodnocovat neskor
			if(s.charAt(i)=='^')	// 	^	opertor umocovania
			{
				l = s.substring(0, i);					// lava strana, pociatocny index 0, dlzka i
				r = s.substring(i+1, s.length());	// prava strana od i+1 po koniec
				left = Vypocitaj_vyraz(l, param, hodnot);
				right = Vypocitaj_vyraz(r, param, hodnot);
				return (float)Math.pow(left,right);
			}
		}
        
/*
RND(x)		vracia nhodn slo z intervalu <0,x)
ABS(x)		vracia absoltnu hodnotu z x
SIN(x)		vracia sin(x)
COS(x)		vracia cos(x)
TAN(x)		vracia tan(x)
COTG(x)		vracia cotan(x)
EXP(x)		vracia e umocnen na x
FLOOR(x)	vracia najvie cel slo menie alebo rovn x
ROUND(x)	zaokrhli x
CEIL(x)		vracia najniie cel slo vie alebo rovn x
SQRT(x)		vracia druh odmocninu z x
LN(x)		vracia prirodzen logaritmus x
LOG(x)		vracia desiatkovy logaritmus x
ASIN(X)		vracia arcsin(x)
ACOS(X)		vracia arccos(x)
ATAN(X)		vracia arctan(x)
*/
        if(s.charAt(s.length()-1)==')' )
        {
            String substr3 = s.substring(0,Math.min(3,s.length()));
            String substr4 = s.substring(0,Math.min(4,s.length()));
            String substr5 = s.substring(0,Math.min(5,s.length()));
            String substr6 = s.substring(0,Math.min(6,s.length()));
            float pom = 0;
            if(substr4.compareTo("rnd(")==0 || substr4.compareTo("RND(")==0)
                return Vypocitaj_vyraz(s.substring(4,s.length()-1), param, hodnot)*(float)Math.random();
            if(substr4.compareTo("abs(")==0 || substr4.compareTo("ABS(")==0)
                return Math.abs(Vypocitaj_vyraz(s.substring(4,s.length()-1), param, hodnot));
            if(substr4.compareTo("sin(")==0 || substr4.compareTo("SIN(")==0)
            {
                pom = (float)Math.sin(PI180*Vypocitaj_vyraz(s.substring(4,s.length()-1), param, hodnot));
                if(pom>0.000000000001) return pom;
                else return 0;
            }
            if(substr4.compareTo("cos(")==0 || substr4.compareTo("COS(")==0)
            {
                pom = (float)Math.cos(PI180*Vypocitaj_vyraz(s.substring(4,s.length()-1), param, hodnot));
                if(pom>0.000000000001) return pom;
                else return 0;
            }
            if(substr4.compareTo("tan(")==0 || substr4.compareTo("TAN(")==0)
            {
                pom = (float)Math.tan(PI180*Vypocitaj_vyraz(s.substring(4,s.length()-1), param, hodnot));
                /*if(pom>0.000000000001)*/ return pom;
                //else return 0;
            }
            if(substr5.compareTo("cotg(")==0 || substr5.compareTo("COTG(")==0)
            {
            	pom = 1.f/(float)Math.tan(PI180*Vypocitaj_vyraz(s.substring(5,s.length()-1), param, hodnot));
                /*if(pom>0.000000000001)*/ return pom;
                //else return 0;
            }
            if(substr4.compareTo("exp(")==0 || substr4.compareTo("EXP(")==0)
            	return (float)Math.exp(Vypocitaj_vyraz(s.substring(4,s.length()-1), param, hodnot));
            if(substr6.compareTo("floor(")==0 || substr6.compareTo("FLOOR(")==0)
            	return (float)Math.floor(Vypocitaj_vyraz(s.substring(6,s.length()-1), param, hodnot));
            if(substr6.compareTo("round(")==0 || substr6.compareTo("ROUND(")==0)
            {
                float vyraz = Vypocitaj_vyraz(s.substring(6,s.length()-1), param, hodnot);
                float absol = Math.abs(vyraz);
                float cela = (float)Math.floor(absol);   // cela cast
                float desat = absol-cela;               // desatinna cast
                if(desat>=0.5f) cela+=1.0f;
                if(vyraz<0) return -cela;
                return cela;
            }
            if(substr5.compareTo("ceil(")==0 || substr5.compareTo("CEIL(")==0)
                return (float)Math.ceil(Vypocitaj_vyraz(s.substring(5,s.length()-1), param, hodnot));
            if(substr5.compareTo("sqrt(")==0 || substr5.compareTo("SQRT(")==0)
                return (float)Math.sqrt(Vypocitaj_vyraz(s.substring(5,s.length()-1), param, hodnot));
            if(substr3.compareTo("ln(")==0 || substr3.compareTo("LN(")==0)
             	return (float)Math.log(Vypocitaj_vyraz(s.substring(3,s.length()-1), param, hodnot));
            if(substr4.compareTo("log(")==0 || substr4.compareTo("LOG(")==0)
                return (float)Math.log(Vypocitaj_vyraz( s.substring(4,s.length()-1), param, hodnot))/(float)Math.log(10);
            if(substr5.compareTo("asin(")==0 || substr5.compareTo("ASIN(")==0)
                return (float)Math.asin(Vypocitaj_vyraz(s.substring(5,s.length()-1), param, hodnot))/PI180;
            if(substr5.compareTo("acos(")==0 || substr5.compareTo("ACOS(")==0)
                return (float)Math.acos(Vypocitaj_vyraz(s.substring(5,s.length()-1), param, hodnot))/PI180;
            if(substr5.compareTo("atan(")==0 || substr5.compareTo("ATAN(")==0 )
                return (float)Math.atan(Vypocitaj_vyraz(s.substring(5,s.length()-1), param, hodnot))/PI180;
        }
		return Vycisli_premennu(s, param, hodnot);
	}
	
	public float Vycisli_premennu(String s, Vector param1, Vector hodnot1) // param - vector string, hodnot - vector float
	{
		Vector param = new Vector(param1);
		Vector hodnot = new Vector(hodnot1);
		
		int j;
		for(j=0; j<param.size(); j++)	// pre vsetky premenne
		{
			String param2 = (String)param.get(j);
			//JOptionPane.showMessageDialog(null,s+" "+param2,"Chyba",JOptionPane.ERROR_MESSAGE);
			if(s.compareTo(/*Math.max(param2.length(),s.length()),*/param2)==0)	// ak sa zhoduju mena
			{
				Float hod = (Float)hodnot.get(j);
				return hod.floatValue();		// vratime hodnotu
			}
		}

		for(j=0; j<cons.size(); j++)	// pre vsetky constanty
		{
			z_constan akt = (z_constan)cons.get(j);
			z_constan cons_pom = (z_constan)cons.get(j);
			if(s.compareTo(cons_pom.name)==0)
			{
				return cons_pom.x;
			}
		}
		
		try			// inak je to cislo
		{
			return Float.valueOf(s).floatValue();
		}
		catch(Exception e)
		{
			return 0.0f;
		}
		//return Float.valueOf(s).floatValue();		
	}
	
	private int Aplikuj_homomorphism()
	{
		boolean pom = je_parametricky;
		int size=0;                 // velkost retazca
		                      
		rh = new StringBuffer();	// uvolnime pamet zabranu rh		

		size = r.length();					// vypocitame potrebnu velkost retazca
		for(int j=0; j<r.length(); j++)		// pre kazdy znak
		{
			char ch=r.charAt(j);
			for(int pr=0; pr<prod_hom.size(); pr++)	// pre vsetky pravidla
			{
				if(((z_production)prod_hom.get(pr)).l==ch) size += ((z_production)prod_hom.get(pr)).r_max_length;	// pripocitame k size velkost pravej strany pravidla-1
			}
		}

		rh.ensureCapacity(size);		// rezervujeme si pamet o velkosti size
	
		int param=0;
		for(int j=0; j<r.length(); j++)				// pre kazdy znak
		{
			if(r.charAt(j)=='(')			// zatvorky preskakujeme, jedna sa o samostatne zatvorky typu "() F", alebo "f(1) ()"
			{
				int zat=1;			// zaciname	za jednou lavou zatvorkov
				j++;
				for(; zat>0; j++)	// prechadzame retazec pokial nanajdeme pravu uzatvaraciu zatvorku
				{
					if(r.charAt(j)=='(')zat++;
					else if(r.charAt(j)==')')zat--;
				}
				// j ukazuje na znak nasledujuci po ')'
				j--;continue;	// j sa nezmeni
			}
			param=0;
			if( j+1<r.length() && r.charAt(j+1)=='(' )		// ak ma dany znak parametre, zistime pocet parametrov
			{
				int k=j+2;
				param++;		// zvysime pocet parametrov ( na 1)
				// spocitame parametre
				int zat=1;			// zaciname za jednou lavou zatvorkov
				for(; zat>0; k++)	// prechadzame retazec pokial nanajdeme parovu uzatvaraciu zatvorku
				{
					if(r.charAt(k)==',')param++;	// kazda ciarka znamena dalsi parameter
					if(r.charAt(k)=='(')zat++;		// zvysime pocet lavych zatvoriek
					else if(r.charAt(k)==')')zat--;	// znizime pocet lavych zatvoriek
				}
			}
			int naslo_sa=0;		// pravidlo
			for(int pr=0; pr<prod_hom.size(); pr++)		// pre vsetky pravidla
			{
				if(((z_production)prod_hom.get(pr)).l == r.charAt(j) && ((z_production)prod_hom.get(pr)).param.size()==param)// pravidlo je mozne pouzit iba
				{									// ak sa zhoduje znak a pocet parametrov
					if(param==0)		// ak nie su parametre
					{
						Vector parametre = new Vector();    // vector Float
						if(((z_production)prod_hom.get(pr)).podmienka.length()==0)
						{
							// pridame retazec vytvoreny z pravej strany pravidla, tak, ze sa vyhodnotia vsetky vyrazi
							rh.append(Vytvor_retazec(((z_production)prod_hom.get(pr)).r, ((z_production)prod_hom.get(pr)).param, parametre)); naslo_sa = 1;
						}
						else if(Vypocitaj_vyraz(((z_production)prod_hom.get(pr)).podmienka, ((z_production)prod_hom.get(pr)).param, parametre)>0 )
						{	rh.append(Vytvor_retazec(((z_production)prod_hom.get(pr)).r, ((z_production)prod_hom.get(pr)).param, parametre)); naslo_sa = 1;}
					}
					else				// su parametre
					{
						int k = j+2,l = j+2;	// k,l ukazuju na naciatok skut. parametrov
						int zat = 1;			// 1 lava zatvorka
						Vector parametre = new Vector();    // vector Foat
						int najdenych=0;        // najdenych parametrov
						for( ; najdenych<param; l++)		// pokial nenajdeme vsetky parametre, l ukazuje na znak
						{
							if(r.charAt(l)=='(') zat++;			// zvysime pocet lavych zatvoriek
							else if(r.charAt(l)==')') zat--;	// znizime pocet lavych zatvoriek
							if(r.charAt(l)==',' || zat==0)		// ak sa najde ',' alebo ukoncovacia zatvorka
							{
								String s = r.substring(k,l);            // string obsahujuci cislo
								parametre.addElement(Float.valueOf(s));	// pridame skutocny parameter
								k=l+1;					// dalsi zacina az za ','
								najdenych++;			// zvysime pocet najdenych parametrov
							}
						}
						if(((z_production)prod_hom.get(pr)).podmienka.length()==0)
						{	rh.append(Vytvor_retazec(((z_production)prod_hom.get(pr)).r, ((z_production)prod_hom.get(pr)).param, parametre)); naslo_sa = 1;}
						else if(Vypocitaj_vyraz(((z_production)prod_hom.get(pr)).podmienka, ((z_production)prod_hom.get(pr)).param, parametre)>0)
						{	rh.append(Vytvor_retazec(((z_production)prod_hom.get(pr)).r, ((z_production)prod_hom.get(pr)).param, parametre)); naslo_sa = 1;}
					}
					if(naslo_sa!=0) break;
				}
			}
			if(naslo_sa==0)     //rh+=r[j];// inak pridame povodny znak, t.j. ak sa nenaslo ziadne pravidlo
			{
				rh.append(r.charAt(j));
				if(param!=0)
				{
					for(int k=j+1; r.charAt(k-1)!=')' && k<r.length(); k++)
					{
						rh.append(r.charAt(k));
					}
				}
			}
		}
		je_parametricky = pom;
		return 1;
	}
}

/*
Subor *.ls obsahuje popis jedneho L systemu
Data su delene na riadky. Kazdy riadok, ktory ma byt nacitany a interpretovany
musi zacinat jednym z tycho klucovych slov (ostatne riadky su ignorovane):
axiom - za nim nasleduje pociatocny retazec
angle - zakladny uhol
define - sluzi na definovanie konstanty
# - za symbolom nasleduje jedno pravidlo. Pravidlo sa sklada z lavej a pravej casti, 
    ktore su oddelene ->
ignore - nasleduj znaky, ktor maj by ignorovan pri testovan kontextu
homomorphism - v alch riadkoch nasleduj pravidl homomorfizmu, ktorch syntax je rovnak ako u pravidiel L-systmu

tieto su zatial ignorovane:
default - nasleduje defincia default parametrov symbolu (u parametrickho L-systmu).

Pre zadany vstup treba:

pre axiom - jeden retazec
pre angle - jeden float
pre define - pole struktur obsahujucich nazov a hodnotu
pre # - pole struktur obsahujucich pravu a lavu sranu

*/